(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     68689,       3155]*)
(*NotebookOutlinePosition[     73216,       3279]*)
(*  CellTagsIndexPosition[     73096,       3272]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The Maple Converter", "Title",
  TextAlignment->Left,
  CellTags->"GeneralInformation"],

Cell["\<\
Chris Willett
Lars Hohmuth
Wolfram Research, Inc.\
\>", "Subsubtitle",
  TextAlignment->Left,
  CellTags->"GeneralInformation"],

Cell["Version 0.95\[Alpha] - April 7, 1998", "Text",
  TextAlignment->Left,
  CellTags->"GeneralInformation"],

Cell[CellGroupData[{

Cell["General Information", "Section",
  CellTags->"GeneralInformation"],

Cell[TextData[{
  "This package is designed to support the conversion of Maple expressions, \
simple worksheets, and procedures into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebooks.  \nSee the text files ",
  StyleBox["sop.txt", "MR"],
  " and ",
  StyleBox["unsop.txt", "MR"],
  " for a listing of the standard Maple functions that are supported and not \
supported, respectively.  There are several functions that support some, but \
not all, options available in Maple.  The canonical examples are the ",
  StyleBox["plot", "MR"],
  " and ",
  StyleBox["dsolve", "MR"],
  " commands. In general, though, most commands, text, and formatting should \
automatically be converted.\n\nThis package is currently in alpha testing. If \
you have any suggestions for added functionality, please email ",
  ButtonBox["convert@wolfram.com",
    ButtonData:>{
      URL[ "mailto: convert@wolfram.com"], None},
    ButtonStyle->"Hyperlink"],
  ButtonBox["."]
}], "Text",
  CellTags->"GeneralInformation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basic Operation and Examples", "Section",
  CellTags->"BasicOperation"],

Cell[TextData[{
  "The Maple to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " converter offers the following functions, which  are also featured in \
the",
  StyleBox[" ", "MR"],
  ButtonBox["ConverterPalette",
    ButtonData:>{
      FrontEnd`FileName[ {
        ParentDirectory[ 
          ParentDirectory[ ]]}, "ConverterPalette.nb", CharacterEncoding -> 
        "WindowsANSI"], None},
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text",
  CellTags->"BasicOperation"],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002f0000GA000`40O003
h00O]P0000000ekg?Nlmk`2b?Nl100000003G_M_Nekg0;5Nm`4mk`400000
00=NmfmkG_L0/Ekg0Cg_0@00000015kgKg]Nmekg[Woo0Ukg0Cg_0@000000
15kgKg]Nmekg[Woo0Ukg0Cg_0@00000015kgKg]Nmekg0Woo00=_Ncg_?Nl0
Z3g_00AooekgG_Lmk`40000000ANmfmkG_MNm`9oo`03Kg]Nmekg0:MNm`05
?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`2WG_L01Cg_
OomNmekg?Nl00@00000015kgKg]Nmekg0Woo00=_NekgG_L0Yekg00Dmkgoo
G_MNmcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg0:MNm`05?Nmooekg
G_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`2WG_L01Cg_OomNmekg
?Nl00@00000015kgKg]Nmekg0Woo00=_NekgG_L0Yekg00DmkgooG_MNmcg_
0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg01aNm`<000=Nm`@0009Nm`<0
0004G_L000000002G_L30003G_L40002G_L50004G_L30003G_L200000ekg
00000002G_L40002G_L50002G_L30002G_L70002G_L300001Ekg00000000
G_L010000ekg100015kg0`007ekg00DmkgooG_MNmcg_0040000000ANmfmk
G_MNm`9oo`03Kg]Nmekg01]Nm`80009Nm`04001Nm`000002G_L20002G_L2
00000ekg00000003G_L30002G_L20003G_L00`00G_MNm`020005G_L20002
G_L00`00G_MNm`02000015kg000005kg0P000Ukg0P000Ukg0P001Ekg0P00
00ANm`00001Nm`80009Nm`80009Nm`800004G_L00000G_L20003G_L01000
G_L000000ekg00<005kgG_L00P000Ukg00<005kgG_L075kg00DmkgooG_MN
mcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg01YNm`8000ENm`80009N
m`80009Nm`800003G_L0000000=Nm`03001Nm`00009Nm`8000INm`8000EN
m`8000ENm`800004G_L00000G_L20002G_L20002G_L20005G_L40004G_L4
0002G_L2000015kg000005kg0P001Ekg0P001Ukg0P008Ekg00DmkgooG_MN
mcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg01YNm`8000ENm`80009N
m`80009Nm`800003G_L00000009Nm`800004G_L00000G_L60002G_L20005
G_L20005G_L2000015kg000005kg0P000Ukg0P000Ukg0P001Ekg0`000ekg
100015kg0P0000ANm`00001Nm`H00003G_L0000000@0009Nm`80025Nm`05
?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`0JG_L20005
G_L20002G_L20002G_L200000ekg00000002G_L2000015kg000005kg0P00
0Ukg0P000Ukg0`0000ANm`00G_MNm`8000ENm`06001Nm`00G_L005kg0P00
0Ukg0P000Ukg0`0000ANm`00G_MNm`@0009Nm`80009Nm`80009Nm`800004
G_L00000G_L20002G_L200000ekg00000002G_L20002G_L2000QG_L01Cg_
OomNmekg?Nl00@00000015kgKg]Nmekg0Woo00=_NekgG_L06Ukg0P000ekg
00<005kgG_L010000Ukg1@000Ukg0`0000ENm`00000005kg00@0009Nm`<0
0004G_L00000G_L50002G_L200000ekg001Nm`0200000ekg000000020002
G_L300000ekg00000002G_L200000ekg00000002G_L50002G_L40003G_L4
0003G_L40002G_L5000OG_L01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg
0Woo00=_NekgG_L06ekg0P000Ukg00<005kgG_L08ekg0P0015kg0P000ekg
0P003ekg0P0035kg0P004ekg0P008Ekg00DmkgooG_MNmcg_0040000000AN
mfmkG_MNm`9oo`03Kg]Nmekg01aNm`@002ENm`8000=Nm`@00003G_L00000
008000eNm`<000]Nm`<001=Nm`80025Nm`05?NmooekgG_Lmk`0100000004
G_M_NekgG_L2Ool00fmkG_MNm`2WG_L01Cg_OomNmekg?Nl00@00000015kg
Kg]Nmekg0Woo00=_NekgG_L0Yekg00DmkgooG_MNmcg_0040000000ANmfmk
G_MNm`9oojY_N`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmk
?Nlmk`2X?Nl017ooG_MNmcg_0@00000015kgKg]Nmekg0Woo00=_NekgG_L0
Yekg00DmkgooG_MNmcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg0:MN
m`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`17G_L5
000fG_L4000GG_L40006G_L01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg
0Woo00=_NekgG_L0B5kg0P00>5kg0P006ekg0P001Ukg00DmkgooG_MNmcg_
0040000000ANmfmkG_MNm`9oo`04Kg]NmekgG_L70003G_L30003G_L<0003
G_L500000ekg000000040002G_L30003G_L400000ekg0000000500001Ekg
00000000G_L01@0000=Nm`0000000Ukg1@000ekg10000Ukg1@000Ukg1@00
0Ukg1P000Ukg10000Ukg0`0000ANm`000000009Nm`8000]Nm`H000YNm`80
00INm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`02
G_L20003G_L01000G_MNmekg0`000Ukg0P000Ukg0P000ekg0P0015kg0P00
00ANm`00001Nm`80009Nm`80009Nm`80009Nm`04001Nm`000003G_L01000
G_L000000ekg00<005kgG_L00P0000ANm`00001Nm`<00003G_L00000009N
m`8000ANm`8000=Nm`04001Nm`000002G_L200000ekg00000002G_L20003
G_L20003G_L20002G_L20002G_L200000ekg00000003G_L2000;G_L6000:
G_L20006G_L01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg0Woo00=_Nekg
G_L00Ukg0P000ekg00L005kgG_MNm`00G_L00002G_L20002G_L20003G_L2
0004G_L2000015kg000005kg0P000Ukg0P000Ukg0P0015kg0P001Ekg0P00
0ekg00@005kgG_MNm`<0009Nm`80009Nm`80009Nm`8000ANm`8000MNm`@0
00=Nm`@000=Nm`8000=Nm`80009Nm`80009Nm`800003G_L0000000=Nm`80
00]Nm`H000YNm`8000INm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2
Ool00fmkG_MNm`02G_L200000ekg001Nm`03G_L200000ekg00000002G_L5
0003G_L20004G_L200000ekg00000002G_L50002G_L20004G_L600001Ekg
000005kg00001Ekg0`000Ukg0P000Ukg0P000Ukg0P0015kg1P0000=Nm`00
00000P000ekg10001Ekg0P000ekg0P000Ukg0P000Ukg0P0000=Nm`000000
0ekg0P002ekg1P002Ukg0P001Ukg00DmkgooG_MNmcg_0040000000ANmfmk
G_MNm`9oo`03Kg]Nmekg009Nm`@000ANm`800003G_L0000000ENm`8000=N
m`8000ANm`800003G_L0000000ENm`80009Nm`8000ANm`80009Nm`800003
G_L00000008000ANm`800004G_L00000G_L300000ekg00000002G_L30000
0ekg001Nm`020002G_L200000ekg00000002G_L200000ekg00000002G_L2
0003G_L20003G_L20002G_L20002G_L200000ekg00000003G_L20004G_L0
0`00G_L00004G_L6000015kg001Nm`001Ukg0P001Ukg00DmkgooG_MNmcg_
0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg009Nm`800005G_L005kg001N
m`0300001Ekg00000000G_L0100015kg0P000ekg0`0000=Nm`0000000Ukg
10000Ukg1@000ekg10000Ukg0P0000ENm`00G_L005kg00<00003G_L00000
00<00003G_L00000009Nm`<00003G_L00000009Nm`@000=Nm`D0009Nm`D0
0003G_L00000008000ANm`@0009Nm`D000ANm`8000ANm`03001Nm`0000AN
m`H00004G_L005kg0006G_L20006G_L01Cg_OomNmekg?Nl00@00000015kg
Kg]Nmekg0Woo00=_NekgG_L00Ukg0P000ekg00<005kgG_L03ekg0P004Ukg
0P002ekg0P000ekg00<005kgG_L0@5kg0P0015kg00<005kg00002ekg00<0
05kg00001Ukg0P001Ukg00DmkgooG_MNmcg_0040000000ANmfmkG_MNm`9o
o`04Kg]NmekgG_L7000?G_L4000BG_L2000:G_L7000]G_L2000CG_L40002
G_L00`00G_L0000;G_L00`00G_L00004G_L40006G_L01Cg_OomNmekg?Nl0
0@00000015kgKg]Nmekg0Woo00=_NekgG_L0Yekg00DmkgooG_MNmcg_0040
000000ANmfmkG_MNm`9oo`03Kg]Nmekg0:MNm`05?NmooekgG_Lmk`010000
0004G_M_NekgG_L2OonZKg/01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg
0Woo00=_Ncg_?Nl0Z3g_00AooekgG_Lmk`40000000ANmfmkG_MNm`9oo`03
Kg]Nmekg0:MNm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmk
G_MNm`2WG_L01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg0Woo00=_Nekg
G_L0@ekg1@00=Ukg10005ekg10002Ukg00DmkgooG_MNmcg_0040000000AN
mfmkG_MNm`9oo`03Kg]Nmekg04ANm`8003QNm`8001]Nm`8000YNm`05?Nmo
oekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`07G_L30003G_L4
0002G_L3000015kg000000000Ukg0`000ekg10000Ukg1@0015kg0`0000=N
m`0000001@0000ENm`00000005kg00D00003G_L00000009Nm`D000=Nm`@0
009Nm`D0009Nm`D0009Nm`H0009Nm`@0009Nm`<00004G_L000000002G_L2
000;G_L6000:G_L2000:G_L01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg
0Woo00=_NekgG_L01Ukg0P000Ukg00@005kg0000009Nm`80009Nm`800003
G_L0000000=Nm`<0009Nm`8000=Nm`03001Nmekg008000ENm`80009Nm`04
001Nm`000003G_L00`00G_MNm`02000015kg000005kg0`0000=Nm`000000
0Ukg0P0015kg0P000ekg00@005kg0000009Nm`800003G_L00000009Nm`80
00=Nm`8000=Nm`80009Nm`80009Nm`800003G_L0000000=Nm`8000]Nm`H0
00YNm`8000YNm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmk
G_MNm`05G_L20005G_L20002G_L20002G_L200000ekg00000003G_L00`00
G_L00002G_L20006G_L20005G_L20004G_L20003G_L01000G_MNmekg0`00
0Ukg0P000Ukg0P000Ukg0P0015kg0P001ekg10000ekg10000ekg0P000ekg
0P000Ukg0P000Ukg0P0000=Nm`0000000ekg0P002ekg1P002Ukg0P002Ukg
00DmkgooG_MNmcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg00ENm`80
00ENm`80009Nm`80009Nm`800003G_L00000009Nm`800004G_L00000G_L6
0002G_L20005G_L20004G_L200000ekg001Nm`04G_L30002G_L20002G_L2
0002G_L20004G_L600000ekg000000020003G_L40005G_L20003G_L20002
G_L20002G_L200000ekg00000003G_L2000;G_L6000:G_L2000:G_L01Cg_
OomNmekg?Nl00@00000015kgKg]Nmekg0Woo00=_NekgG_L01Ekg0P001Ekg
0P000Ukg0P000Ukg0P0000=Nm`0000000Ukg0P0000ANm`00001Nm`80009N
m`80009Nm`<00004G_L005kgG_L20004G_L40004G_L2000015kg000005kg
0`0000=Nm`0000000Ukg0`0000=Nm`00G_L00P000Ukg0P0000=Nm`000000
0Ukg0P0000=Nm`0000000Ukg0P000ekg0P000ekg0P000Ukg0P000Ukg0P00
00=Nm`0000000ekg0P0015kg00<005kg000015kg1P0000ANm`00G_L000IN
m`8000YNm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MN
m`05G_L20003G_L00`00G_MNm`040002G_L50002G_L300001Ekg00000000
G_L010000Ukg0`0000ANm`00001Nm`D0009Nm`800005G_L005kg001Nm`03
00000ekg0000000300000ekg00000002G_L300000ekg00000002G_L40003
G_L50002G_L500000ekg000000020004G_L40002G_L50004G_L20004G_L0
0`00G_L00004G_L6000015kg001Nm`001Ukg0P002Ukg00DmkgooG_MNmcg_
0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg00INm`80009Nm`03001Nmekg
02=Nm`8000ANm`8000=Nm`03001Nmekg041Nm`8000ANm`03001Nm`0000]N
m`03001Nm`0000INm`8000YNm`05?NmooekgG_Lmk`0100000004G_M_Nekg
G_L2Ool00fmkG_MNm`07G_L4000UG_L20003G_L7000]G_L2000CG_L40002
G_L00`00G_L0000;G_L00`00G_L00004G_L4000:G_L01Cg_OomNmekg?Nl0
0@00000015kgKg]Nmekg0Woo00=_NekgG_L0Yekg00DmkgooG_MNmcg_0040
000000ANmfmkG_MNm`9oo`03Kg]Nmekg0:MNm`05?NmooekgG_Lmk`010000
0004G_M_NekgG_L2OonZKg/01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg
0Woo00=_Ncg_?Nl0Z3g_00AooekgG_Lmk`40000000ANmfmkG_MNm`9oo`03
Kg]Nmekg0:MNm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmk
G_MNm`2WG_L01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg0Woo00=_Nekg
G_L0Yekg00DmkgooG_MNmcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg
0:MNm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`0?
G_L600001Ekg00000000G_L02@000ekg0`000Ukg1P000Ukg300000=Nm`00
000010000Ukg1@000Ukg100015kg0`000ekg10000Ukg0`0000ANm`000000
009Nm`<000=Nm`@0009Nm`D000ANm`<000=Nm`@0009Nm`D001ENm`05?Nmo
oekgG_Lmk`0100000004G_M_NekgG_L2Ool00fmkG_MNm`0AG_L20004G_L2
00000ekg00000003G_L20004G_L20002G_L01000G_MNmekg0P000ekg0P00
0Ukg0P000ekg0P001Ekg0P0015kg0P000Ukg00@005kg000000=Nm`03001N
mekg0080009Nm`04001Nm`000002G_L20002G_L200000ekg00000003G_L3
0002G_L20003G_L00`00G_MNm`020005G_L20002G_L01000G_L000000ekg
00<005kgG_L00P005ekg00DmkgooG_MNmcg_0040000000ANmfmkG_MNm`9o
o`03Kg]Nmekg015Nm`8000ANm`800003G_L0000000=Nm`8000ANm`8000IN
m`8000=Nm`80009Nm`8000=Nm`8000ENm`8000ENm`8000=Nm`8000ENm`80
00ENm`80009Nm`80009Nm`800003G_L0000000=Nm`03001Nm`00009Nm`80
00INm`8000ENm`8000ANm`8000INm`8001MNm`05?NmooekgG_Lmk`010000
0004G_M_NekgG_L2Ool00fmkG_MNm`0AG_L20004G_L200000ekg00000003
G_L20004G_L20006G_L20004G_L50003G_L20005G_L20006G_L20002G_L6
00000ekg00000005G_L20002G_L20002G_L200000ekg00000002G_L20000
15kg000005kg1P000Ukg0P001Ekg0P0015kg1P000Ukg0P005ekg00Dmkgoo
G_MNmcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg015Nm`8000ANm`80
0003G_L0000000=Nm`8000ANm`8000INm`8000MNm`8000=Nm`8000ENm`80
00ANm`03001Nmekg00800003G_L00000009Nm`800003G_L0000000ENm`80
009Nm`80009Nm`800003G_L00000009Nm`800004G_L00000G_L20002G_L2
0002G_L3000015kg001Nmekg0P0015kg0P000Ukg0P000Ukg0`0000=Nm`00
G_L04ekg00DmkgooG_MNmcg_0040000000ANmfmkG_MNm`9oo`03Kg]Nmekg
015Nm`8000=Nm`D0009Nm`@000=Nm`D0009Nm`@000ANm`@000ANm`8000=N
m`@000ANm`D0009Nm`@0009Nm`8000=Nm`03001Nmekg00@0009Nm`D0009N
m`<00005G_L00000001Nm`040002G_L3000015kg000005kg1@000ekg1000
0Ukg0`0000=Nm`00000055kg00DmkgooG_MNmcg_0040000000ANmfmkG_MN
m`9oo`03Kg]Nmekg015Nm`80019Nm`8001ANm`8001UNm`80009Nm`03001N
mekg02=Nm`8002ENm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2Ool0
0fmkG_MNm`0?G_L6000:G_L20004G_L20006G_L2000:G_L40005G_L2000C
G_L4000UG_L2000UG_L01Cg_OomNmekg?Nl00@00000015kgKg]Nmekg0Woo
00=_NekgG_L0Yekg00DmkgooG_MNmcg_0040000000ANmfmkG_MNm`9oo`03
Kg]Nmekg0:MNm`05?NmooekgG_Lmk`0100000004G_M_NekgG_L2OonZKg/0
1Cg_OomNmekg?Nl00@00000015kgKg]Nmekg[Woo0Ukg0Cg_0@0000000ekg
Kg]Nm`2aG_L1?Nl100000004G_M_NekgG_N^00l2G_L1?Nl100000004G_M_
NekgG_N^00l2G_L1?Nl100000004G_M_NekgG_NQ00l;000200l2G_L1?Nl1
00000004G_M_NekgG_L400l4Ool300l3Ool200l01Goo00l03`0?Ool00`0?
00=oo`0?00l00P0?0goo0P0?00Eoo`0?00l03goo00803`=oo`803`04Ool0
3`0?Ool700l4Ool0100?Ool03`0?0goo0`0?00Aoo`0?00moo`803`=oo`80
3`04Ool03`0?Ool300l00goo00moo`03Oom200l00fmk?Nlmk`07?Nl00`00
00l03`02G_L1?Nl100000004G_M_NekgG_L300l00goo00l03`0200l00goo
00moo`0300l00goo00moo`0300l01Goo00l03`0?Ool00`0?00Moo`0?00l0
3goo00moo`0200l017oo00l03goo0`0?00=oo`0?Ool00P0?00=oo`0?00l0
100?00Uoo`0?00l03goo00moo`0?Ool00`0?00=oo`0?Ool00P0?00Aoo`0?
00moo`<03`03Ool03`0?00803`07Ool03`0?00moo`0?Ool00`0?00=oo`0?
00l0?`0?00=_NekgG_L01Ukg00@mk`0000l03`9Nm`4mk`40000000ANmfmk
G_MNm`<03`03Ool03`0?00@03`07Ool03`0?00moo`0?Ool00`0?00Ioo`0?
00moo`0?Ool200l00goo00l03`0300l017oo00l03goo0P0?00=oo`0?00l0
0`0?00Aoo`0?00moo`H03`09Ool03`0?00moo`0?Ool03goo00D03`04Ool0
3`0?Ool200l00goo00l03`0600l01goo00l03`0?Ool03goo00<03`03Ool0
3`0?03l03`04Kg]Nm`000002G_L200000ekg?Nl0000200l2G_L1?Nl10000
0004G_M_NekgG_L300l00goo00l03`0400l01goo00l03`0?Ool03goo00<0
3`06Ool03`0?Ool03goo0P0?1Goo00D03goo00l03goo00803`Eoo`0500mo
o`0?00moo`0700l4Ool00`0?Ool03`05Ool01@0?Ool03`0?Ool00P0?1Goo
100?00Moo`0?00l03goo00moo`0300l00goo00l03`0o00l00fmkG_MNm`04
0002G_L013g_00003`0?0Ukg0Cg_0@00000015kgKg]Nmekg0`0?00=oo`0?
00l0100?00Ioo`0?00l03goo00l2Ool200l00goo00moo`0300l00goo00mo
o`0300l00goo00moo`0200l017oo00l03goo0`0?00=oo`0?Ool00P0?1Goo
1P0?00Eoo`0?Ool03goo00<03`03Ool03goo00803`04Ool03`0?Ool300l0
0goo00l03`0200l2Ool200l00goo00moo`0300l00goo00l03`0o00l016mk
G_MNmekg0P000ekg00@mk`0000l03`9Nm`4mk`40000000ANmfmkG_MNm`<0
3`03Ool03`0?00D03`=oo`803`04Ool03gooOol200l01Goo00l03`0?Ool0
0P0?0goo0P0?0Woo00<03gooOol00P0?0goo0P0?0Woo00<03goo00l00`0?
00=oo`0?00l00goo0P0?00=oo`0?00l00goo0P0?0Woo00<03gooOol00P0?
0goo1@0?00Aoo`0?Oomoo`803`Aood803`03Kg]Nmekg00@0009Nm`04?Nl0
000?00l2G_L1?Nl100000004G_M_NekgG_L300l00goo00l03`0O00l00goo
00l03`0900l00goo00l03`0200l00goo00l03`0500l00goo00l03`0500l0
17oo00l03goo4@0?00=oo`0?00l0@`0?00A_Nekg0000009Nm`800003G_Lm
k`0000803`9Nm`4mk`40000000ANmfmkG_MNm`<03`03Ool03`0?00803`03
Ool03`0?01X03`03Ool03`0?00T03`03Ool03`0?00803`03Ool03`0?00D0
3`03Ool03`0?00D03`04Ool03`0?OolA00l00goo00l03`1300l00fmkG_MN
m`06G_L013g_00003`0?0Ukg0Cg_0@00000015kgKg]Nmekg100?17oo:@0?
1Goo200?00=oo`0?00l06P0?00=oo`0?00l0@`0?00=_NekgG_L01Ukg00@m
k`0000l03`9Nm`4mk`40000000ANmfmkG_MNmj403`Y_N`0300003`0?009N
m`4mk`40000000ANmfmkG_MNmjh03`9Nm`4mk`40000000ANmfmkG_MNmjh0
3`9Nm`4mk`40000000=NmfmkG_L0/Ekg0Cg_0@0000000ekgKg]Nm`2aG_L1
?Nl100000003G_M_Nfmk0;5_N`4mk`400000]Ekg0@000000\
\>"], "Picture",
  ImageSize->{182, 93},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->"BasicOperation"],

Cell["\<\
To use the palette, you must either click the InitializeConverter button on \
the palette or load the package.\
\>", "Text",
  CellTags->"BasicOperation"],

Cell["This loads the Maple Converter package.", "Commentary",
  CellTags->"BasicOperation"],

Cell[BoxData[
    \(<< MapleConvert`MTrans`\)], "Input",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{

Cell["ConvertExpression", "Subsection",
  CellTags->"BasicOperation"],

Cell[TextData[{
  StyleBox["ConvertExpression[", "MR",
    FontFamily->"Courier"],
  StyleBox["]",
    FontFamily->"Courier"],
  " translates the Maple command into an appropriate function and returns it \
in an Input cell.  "
}], "Text",
  CellTags->"BasicOperation"],

Cell["\<\
This is an example of a Maple command translated to Mathematica input.\
\>", "Commentary",
  CellTags->"BasicOperation"],

Cell[BoxData[
    \(ConvertExpression["\<sin(z)+cos(z)\>"]\)], "Input",
  CellTags->"BasicOperation"],

Cell[BoxData[
    \(Sin[z] + Cos[z]\)], "Input",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["EvaluateExpression", "Subsection",
  CellTags->"BasicOperation"],

Cell[TextData[{
  StyleBox["EvaluateExpression[]", "MR"],
  " ",
  "translates the Maple command into an appropriate ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function, evaluates it, and returns the result in an Output cell. This \
feature works only if the translation supplies all the arguments and \
parameters. If you get error messages, use ",
  StyleBox["ConvertExpression[]", "MR"],
  " ",
  StyleBox["and fill in the parameters before evaluating.",
    FontFamily->"Times New Roman"]
}], "Text",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      StyleBox["EvaluateExpression",
        FontFamily->"Courier",
        FontWeight->"Bold"], "[", "\"\<sin(z)+cos(z)\>\"", "]"}]], "Input",
  CellTags->"BasicOperation"],

Cell[BoxData[
    \(Cos[z] + Sin[z]\)], "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      StyleBox["EvaluateExpression",
        FontFamily->"Courier",
        FontWeight->"Bold"], "[", "\"\<simplify(log(exp(1),x)=ln(x))\>\"", 
      "]"}]], "Input",
  CellTags->"BasicOperation"],

Cell[BoxData[
    \(True\)], "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      StyleBox["EvaluateExpression",
        FontFamily->"Courier",
        FontWeight->"Bold"], "[", "\"\<plot(sin(z),z=0..5,y=-1..1)\>\"", "]"}]
      ], "Input",
  CellTags->"BasicOperation"],

Cell[GraphicsData["PostScript", "\<\

%!

%%Creator: Mathematica

%%AspectRatio: .61803 

MathPictureStart

/Mabs {

Mgmatrix idtransform

Mtmatrix dtransform

} bind def

/Mabsadd { Mabs

3 -1 roll add

3 1 roll add

exch } bind def

%% Graphics

%%IncludeResource: font Courier

%%IncludeFont: Courier

/Courier findfont 10  scalefont  setfont

% Scaling calculations

0.0238095 0.190476 0.309017 0.309017 [

[.21429 .29652 -3 -9 ]

[.21429 .29652 3 0 ]

[.40476 .29652 -3 -9 ]

[.40476 .29652 3 0 ]

[.59524 .29652 -3 -9 ]

[.59524 .29652 3 0 ]

[.78571 .29652 -3 -9 ]

[.78571 .29652 3 0 ]

[.97619 .29652 -3 -9 ]

[.97619 .29652 3 0 ]

[.01131 0 -12 -4.5 ]

[.01131 0 0 4.5 ]

[.01131 .07725 -30 -4.5 ]

[.01131 .07725 0 4.5 ]

[.01131 .15451 -24 -4.5 ]

[.01131 .15451 0 4.5 ]

[.01131 .23176 -30 -4.5 ]

[.01131 .23176 0 4.5 ]

[.01131 .38627 -24 -4.5 ]

[.01131 .38627 0 4.5 ]

[.01131 .46353 -18 -4.5 ]

[.01131 .46353 0 4.5 ]

[.01131 .54078 -24 -4.5 ]

[.01131 .54078 0 4.5 ]

[.01131 .61803 -6 -4.5 ]

[.01131 .61803 0 4.5 ]

[ 0 0 0 0 ]

[ 1 .61803 0 0 ]

] MathScale

% Start of Graphics

1 setlinecap

1 setlinejoin

newpath

0 g

.25 Mabswid

.21429 .30902 m

.21429 .31527 L

s

[(1)] .21429 .29652 0 1 Mshowa

.40476 .30902 m

.40476 .31527 L

s

[(2)] .40476 .29652 0 1 Mshowa

.59524 .30902 m

.59524 .31527 L

s

[(3)] .59524 .29652 0 1 Mshowa

.78571 .30902 m

.78571 .31527 L

s

[(4)] .78571 .29652 0 1 Mshowa

.97619 .30902 m

.97619 .31527 L

s

[(5)] .97619 .29652 0 1 Mshowa

.125 Mabswid

.0619 .30902 m

.0619 .31277 L

s

.1 .30902 m

.1 .31277 L

s

.1381 .30902 m

.1381 .31277 L

s

.17619 .30902 m

.17619 .31277 L

s

.25238 .30902 m

.25238 .31277 L

s

.29048 .30902 m

.29048 .31277 L

s

.32857 .30902 m

.32857 .31277 L

s

.36667 .30902 m

.36667 .31277 L

s

.44286 .30902 m

.44286 .31277 L

s

.48095 .30902 m

.48095 .31277 L

s

.51905 .30902 m

.51905 .31277 L

s

.55714 .30902 m

.55714 .31277 L

s

.63333 .30902 m

.63333 .31277 L

s

.67143 .30902 m

.67143 .31277 L

s

.70952 .30902 m

.70952 .31277 L

s

.74762 .30902 m

.74762 .31277 L

s

.82381 .30902 m

.82381 .31277 L

s

.8619 .30902 m

.8619 .31277 L

s

.9 .30902 m

.9 .31277 L

s

.9381 .30902 m

.9381 .31277 L

s

.25 Mabswid

0 .30902 m

1 .30902 L

s

.02381 0 m

.03006 0 L

s

[(-1)] .01131 0 1 0 Mshowa

.02381 .07725 m

.03006 .07725 L

s

[(-0.75)] .01131 .07725 1 0 Mshowa

.02381 .15451 m

.03006 .15451 L

s

[(-0.5)] .01131 .15451 1 0 Mshowa

.02381 .23176 m

.03006 .23176 L

s

[(-0.25)] .01131 .23176 1 0 Mshowa

.02381 .38627 m

.03006 .38627 L

s

[(0.25)] .01131 .38627 1 0 Mshowa

.02381 .46353 m

.03006 .46353 L

s

[(0.5)] .01131 .46353 1 0 Mshowa

.02381 .54078 m

.03006 .54078 L

s

[(0.75)] .01131 .54078 1 0 Mshowa

.02381 .61803 m

.03006 .61803 L

s

[(1)] .01131 .61803 1 0 Mshowa

.125 Mabswid

.02381 .01545 m

.02756 .01545 L

s

.02381 .0309 m

.02756 .0309 L

s

.02381 .04635 m

.02756 .04635 L

s

.02381 .0618 m

.02756 .0618 L

s

.02381 .09271 m

.02756 .09271 L

s

.02381 .10816 m

.02756 .10816 L

s

.02381 .12361 m

.02756 .12361 L

s

.02381 .13906 m

.02756 .13906 L

s

.02381 .16996 m

.02756 .16996 L

s

.02381 .18541 m

.02756 .18541 L

s

.02381 .20086 m

.02756 .20086 L

s

.02381 .21631 m

.02756 .21631 L

s

.02381 .24721 m

.02756 .24721 L

s

.02381 .26266 m

.02756 .26266 L

s

.02381 .27812 m

.02756 .27812 L

s

.02381 .29357 m

.02756 .29357 L

s

.02381 .32447 m

.02756 .32447 L

s

.02381 .33992 m

.02756 .33992 L

s

.02381 .35537 m

.02756 .35537 L

s

.02381 .37082 m

.02756 .37082 L

s

.02381 .40172 m

.02756 .40172 L

s

.02381 .41717 m

.02756 .41717 L

s

.02381 .43262 m

.02756 .43262 L

s

.02381 .44807 m

.02756 .44807 L

s

.02381 .47898 m

.02756 .47898 L

s

.02381 .49443 m

.02756 .49443 L

s

.02381 .50988 m

.02756 .50988 L

s

.02381 .52533 m

.02756 .52533 L

s

.02381 .55623 m

.02756 .55623 L

s

.02381 .57168 m

.02756 .57168 L

s

.02381 .58713 m

.02756 .58713 L

s

.02381 .60258 m

.02756 .60258 L

s

.25 Mabswid

.02381 0 m

.02381 .61803 L

s

0 0 m

1 0 L

1 .61803 L

0 .61803 L

closepath

clip

newpath

.5 Mabswid

.02381 .30902 m

.06244 .37127 L

.10458 .43616 L

.14415 .49152 L

.18221 .53738 L

.20342 .5591 L

.22272 .57618 L

.24299 .59117 L

.26171 .60217 L

.28086 .6105 L

.29094 .61367 L

.29653 .61505 L

.30164 .61609 L

.30646 .61687 L

.31105 .61743 L

.31356 .61765 L

.31589 .61782 L

.31721 .61789 L

.31845 .61795 L

.31913 .61797 L

.31988 .61799 L

.32117 .61802 L

.32232 .61803 L

.32338 .61803 L

.32455 .61802 L

.32581 .618 L

.32712 .61796 L

.32837 .61791 L

.33071 .61778 L

.33289 .61762 L

.33491 .61743 L

.33947 .61688 L

.34411 .61614 L

.3491 .61514 L

.35814 .61279 L

.36889 .60911 L

.37874 .6049 L

.39766 .5946 L

.41837 .58011 L

.45893 .54264 L

.49797 .49658 L

.53947 .43907 L

.57945 .3778 L

.61792 .31598 L

.65884 .24996 L

.69824 .18892 L

.73613 .13503 L

.77647 .08523 L

.81529 .04673 L

.83504 .03123 L

Mistroke

.85657 .01773 L

.86656 .01272 L

.8772 .00829 L

.88724 .00496 L

.89633 .00267 L

.90122 .00173 L

.90638 .00096 L

.91078 .00048 L

.91328 .00028 L

.91561 .00014 L

.91691 9e-005 L

.91759 6e-005 L

.91832 4e-005 L

.91895 3e-005 L

.91965 1e-005 L

.92088 0 L

.92207 0 L

.92335 2e-005 L

.92457 4e-005 L

.92567 8e-005 L

.92698 .00013 L

.92818 .0002 L

.93093 .00039 L

.9336 .00063 L

.93645 .00096 L

.94156 .00173 L

.94627 .00263 L

.9569 .00535 L

.96628 .00853 L

.97619 .01269 L

Mfstroke

% End of Graphics

MathPictureEnd


\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40O003h00Oogoo8Goo003oOolQ
Ool00?moob5oo`00ogoo8Goo003oOolQOol00?moob5oo`006Goo1@00ogoo0goo000KOol00`00Oomo
o`3oOol3Ool001Eoo`<000=oo`03001oogoo00=oo`800>1oo`l000ioo`006goo00<007ooOol00goo
00<007ooOol0gGoo0P003goo0`002goo000KOol00`00Oomoo`03Ool00`00Oomoo`3JOol3000DOol2
0009Ool001Yoo`8000Eoo`03001oogoo0=Moo`<001Uoo`8000Moo`008Goo0P00eWoo0P009Goo000Q
Ool00`00Oomoo`3DOol00`00Oomoo`0UOol0025oo`03001oogoo0=9oo`8002Qoo`008Goo00<007oo
Ool0dGoo00<007ooOol0:7oo000QOol2003AOol00`00Oomoo`0YOol0025oo`03001oogoo0<ioo`80
02aoo`008Goo00<007ooOol0cGoo00<007ooOol0;7oo000QOol00`00Oomoo`3;Ool2000_Ool0025o
o`800<]oo`03001oogoo02moo`008Goo00<007ooOol0bGoo00<007ooOol0<7oo000QOol00`00Oomo
o`38Ool00`00Oomoo`0aOol0025oo`03001oogoo0<Moo`03001oogoo039oo`008Goo0P00agoo00<0
07ooOol0<goo000QOol00`00Oomoo`35Ool00`00Oomoo`0dOol000aoo`03001oogoo009oo`03001o
ogoo009oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo0<Aoo`03001oogoo03Eoo`002goo
00<007oo000027oo00L007ooOomoo`00Ool00004Ool00`00Oomoo`33Ool00`00Oomoo`0fOol000Io
o`<0009oo`03001oo`0000Qoo`03001oogoo00=oo`03001oogoo009oo`800<=oo`03001oogoo03Mo
o`002goo00<007oo00002Goo00<007ooOol00P001Goo00<007ooOol0`Goo00<007ooOol0>7oo000;
Ool00`00Ool00007Ool00`00Ool00002Ool00`00Oomoo`04Ool00`00Oomoo`30Ool00`00Oomoo`0i
Ool000aoo`03001oogoo00Ioo`<0009oo`<000Aoo`03001oogoo0;moo`03001oogoo03Yoo`008Goo
0P00_goo00<007ooOol0>goo000QOol00`00Oomoo`2nOol00`00Oomoo`0kOol0025oo`03001oogoo
0;eoo`03001oogoo03aoo`008Goo00<007ooOol0_7oo00<007ooOol0?Goo000QOol2002lOol00`00
Oomoo`0nOol0025oo`03001oogoo0;]oo`03001oogoo03ioo`008Goo00<007ooOol0^Woo00<007oo
Ool0?goo000QOol00`00Oomoo`2iOol00`00Oomoo`10Ool0025oo`800;Uoo`03001oogoo045oo`00
8Goo00<007ooOol0^7oo00<007ooOol0@Goo000QOol00`00Oomoo`2gOol00`00Oomoo`12Ool0025o
o`03001oogoo0;Ioo`03001oogoo04=oo`008Goo0P00]Woo00<007ooOol0A7oo000QOol00`00Oomo
o`2eOol00`00Oomoo`14Ool0015oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00=oo`03
001oogoo0;Aoo`03001oogoo04Eoo`0047oo00<007oo00001goo00<007oo000017oo00<007ooOol0
/goo00<007ooOol0AWoo000;Ool30002Ool00`00Ool00009Ool00`00Oomoo`02Ool2002cOol00`00
Oomoo`17Ool0011oo`03001oo`0000Moo`8000Eoo`03001oogoo0;9oo`03001oogoo04Moo`0047oo
00<007oo00001goo00<007ooOol017oo00<007ooOol0/Goo00<007ooOol0B7oo000AOol00`00Oomo
o`06Ool30004Ool00`00Oomoo`2`Ool00`00Oomoo`19Ool0025oo`800;5oo`03001oogoo04Uoo`00
8Goo00<007ooOol0[goo00<007ooOol0BWoo000QOol00`00Oomoo`2^Ool00`00Oomoo`1;Ool0025o
o`03001oogoo0:eoo`03001oogoo04aoo`008Goo0P00[Woo00<007ooOol0C7oo000QOol00`00Oomo
o`2/Ool00`00Oomoo`1=Ool0025oo`03001oogoo0:]oo`03001oogoo04ioo`008Goo00<007ooOol0
Zgoo00<007ooOol0CWoo000QOol2002[Ool00`00Oomoo`1?Ool0025oo`03001oogoo0:Uoo`03001o
ogoo051oo`008Goo00<007ooOol0ZGoo00<007ooOol0D7oo000QOol00`00Oomoo`2XOol00`00Oomo
o`1AOol0025oo`800:Uoo`03001oogoo055oo`008Goo00<007ooOol0Ygoo00<007ooOol0DWoo000<
Ool00`00Oomoo`02Ool01000Oomoogoo0`000goo00<007ooOol00goo00<007ooOol0YWoo00<007oo
Ool0Dgoo000;Ool00`00Ool00007Ool00`00Oomoo`02Ool00`00Ool00004Ool00`00Oomoo`2VOol0
0`00Oomoo`1COol000Ioo`<0009oo`03001oo`0000Qoo`03001oogoo00=oo`03001oogoo009oo`80
0:Ioo`03001oogoo05Aoo`002goo00<007oo000027oo00@007ooOomoo`8000Eoo`03001oogoo0:Ao
o`03001oogoo05Eoo`002goo00<007oo00001goo00<007oo00000Woo00<007ooOol017oo00<007oo
Ool0Y7oo00<007ooOol0EGoo000<Ool00`00Oomoo`07Ool01000Oomoogoo0`0017oo00<007ooOol0
Xgoo00<007ooOol0EWoo000QOol2002TOol00`00Oomoo`1FOol0025oo`03001oogoo0:9oo`03001o
ogoo05Moo`008Goo00<007ooOol0XGoo00<007ooOol0F7oo000QOol00`00Oomoo`2QOol00`00Oomo
o`1HOol0025oo`800:5oo`03001oogoo05Uoo`008Goo00<007ooOol0;7oo1@00;Goo0`00;goo0P00
3Goo00<007ooOol087oo0`00;goo00<007ooOol01Goo000QOol00`00Oomoo`0^Ool00`00Oomoo`0]
Ool00`00Oomoo`0aOol00`00Oomoo`0:Ool00`00Oomoo`0QOol00`00Oomoo`0]Ool00`00Ool00006
Ool0025oo`03001oogoo02ioo`03001oogoo02ioo`03001oogoo02moo`03001oogoo00Yoo`03001o
ogoo021oo`@0031oo`03001oogoo00Aoo`008Goo0P00;goo00<007ooOol0;Woo00<007ooOol0<7oo
00<007ooOol027oo00<007ooOol08Goo00<007oo0000;goo0P001goo000QOol00`00Oomoo`0^Ool0
0`00Oomoo`0]Ool00`00Ool0000_Ool00`00Ool0000:Ool00`00Oomoo`0ROol2000_Ool00`00Oomo
o`06Ool0025oo`03001oogoo02eoo`80031oo`03001oogoo02moo`03001oogoo00Qoo`03001oogoo
02Aoo`03001oogoo02eoo`<000Ioo`008Goo00<007ooOol0W7oo00<007ooOol0GGoo000QOol2002L
Ool00`00Oomoo`1NOol0025oo`03001oogoo09Yoo`03001oogoo05moo`008Goo00<007ooOol0VWoo
00<007ooOol0Ggoo000QOol00`00Oomoo`2IOol00`00Oomoo`1POol001]oool000H000008Goo0P00
27oo00<007ooOol01goo00<007ooOol01Woo00<007ooOol01goo00<007ooOol01goo00<007ooOol0
1goo00<007ooOol01goo00<007ooOol01goo00<007ooOol01goo00<007ooOol01goo00<007ooOol0
1goo00<007ooOol01goo00<007ooOol01Woo00<007ooOol01goo00<007ooOol01goo00<007ooOol0
17oo00@007ooOol000Uoo`03001oogoo00Moo`03001oogoo00Moo`03001oogoo00Moo`03001oogoo
00Moo`03001oogoo00Moo`03001oogoo00Moo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo
00Eoo`008Goo0P00;goo00<007ooOol0;goo00<007ooOol0;Woo00<007ooOol00goo00<007ooOol0
:Goo00<007ooOol0;Woo00<007ooOol01Goo000QOol00`00Ool0002FOol00`00Oomoo`1SOol0025o
o`800003Ool007oo09Aoo`03001oogoo06=oo`008Goo00@007ooOol009Aoo`03001oogoo06Aoo`00
8Goo00D007ooOomoo`00099oo`03001oogoo06Eoo`008Goo00D007ooOomoo`00099oo`03001oogoo
06Eoo`008Goo0P000goo00<007ooOol0SWoo00<007ooOol0IWoo000QOol00`00Oomoo`03Ool00`00
Oomoo`2=Ool00`00Oomoo`1VOol0025oo`03001oogoo00=oo`03001oogoo08aoo`03001oogoo06Mo
o`008Goo00<007ooOol017oo00<007ooOol0RWoo00<007ooOol0J7oo000QOol20006Ool00`00Oomo
o`29Ool00`00Oomoo`1XOol0025oo`03001oogoo00Eoo`03001oogoo08Qoo`03001oogoo06Uoo`00
8Goo00<007ooOol01Woo00<007ooOol0QWoo00<007ooOol0JWoo000QOol00`00Oomoo`06Ool00`00
Oomoo`26Ool00`00Oomoo`1ZOol0025oo`8000Qoo`03001oogoo08Aoo`03001oogoo06]oo`008Goo
00<007ooOol027oo00<007ooOol0Pgoo00<007ooOol0Jgoo000<Ool00`00Oomoo`02Ool01000Oomo
ogoo0`000goo00<007ooOol00goo00<007ooOol027oo00<007ooOol0PWoo00<007ooOol0K7oo000;
Ool00`00Ool00007Ool00`00Oomoo`02Ool00`00Ool00004Ool00`00Oomoo`09Ool00`00Oomoo`20
Ool00`00Oomoo`1]Ool000]oo`03001oo`0000Qoo`03001oogoo00=oo`03001oogoo009oo`8000]o
o`03001oogoo07moo`03001oogoo06eoo`002goo00<007oo000027oo00@007ooOomoo`8000Eoo`03
001oogoo00Yoo`03001oogoo07ioo`03001oogoo06ioo`002goo00<007oo00001goo00<007oo0000
0Woo00<007ooOol017oo00<007ooOol02goo00<007ooOol0O7oo00<007ooOol0Kgoo000<Ool00`00
Oomoo`07Ool01000Oomoogoo0`0017oo00<007ooOol037oo00<007ooOol0Ngoo00<007ooOol0Kgoo
000QOol2000=Ool00`00Oomoo`1jOol00`00Oomoo`1`Ool0025oo`03001oogoo00eoo`03001oogoo
07Uoo`03001oogoo071oo`008Goo00<007ooOol03Goo00<007ooOol0N7oo00<007ooOol0LGoo000Q
Ool00`00Oomoo`0>Ool00`00Oomoo`1fOol00`00Oomoo`1bOol0025oo`80011oo`03001oogoo07Eo
o`03001oogoo079oo`008Goo00<007ooOol03goo00<007ooOol0M7oo00<007ooOol0Lgoo000QOol0
0`00Oomoo`0@Ool00`00Oomoo`1bOol00`00Oomoo`1dOol0025oo`03001oogoo015oo`03001oogoo
075oo`03001oogoo07Aoo`008Goo0P004Woo00<007ooOol0L7oo00<007ooOol0MGoo000QOol00`00
Oomoo`0BOol00`00Oomoo`1_Ool00`00Oomoo`1eOol0025oo`03001oogoo01=oo`03001oogoo06eo
o`03001oogoo07Ioo`008Goo00<007ooOol04goo00<007ooOol0K7oo00<007ooOol0Mgoo000QOol2
000EOol00`00Oomoo`1[Ool00`00Oomoo`1gOol0025oo`03001oogoo01Eoo`03001oogoo06Uoo`03
001oogoo07Qoo`004Goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
5Woo00<007ooOol0Igoo00<007ooOol0NGoo000@Ool00`00Ool00007Ool00`00Ool00004Ool00`00
Oomoo`0FOol00`00Oomoo`1VOol00`00Oomoo`1jOol0011oo`03001oo`0000Uoo`03001oogoo009o
o`8001Qoo`03001oogoo06Eoo`03001oogoo07Yoo`0047oo00<007oo00001goo0P001Goo00<007oo
Ool067oo00<007ooOol0Hgoo00<007ooOol0Ngoo000@Ool00`00Ool00007Ool00`00Oomoo`04Ool0
0`00Oomoo`0HOol00`00Oomoo`1ROol00`00Oomoo`1lOol0015oo`03001oogoo00Ioo`<000Aoo`03
001oogoo01Uoo`03001oogoo061oo`03001oogoo07eoo`008Goo0P006goo00<007ooOol0Ggoo00<0
07ooOol0OGoo000QOol00`00Oomoo`0KOol00`00Oomoo`1MOol00`00Oomoo`1nOol0025oo`03001o
ogoo01]oo`03001oogoo05aoo`03001oogoo07moo`008Goo00<007ooOol077oo00<007ooOol0FWoo
00<007ooOol0P7oo000QOol2000NOol00`00Oomoo`1IOol00`00Oomoo`20Ool0025oo`03001oogoo
01ioo`03001oogoo05Moo`03001oogoo085oo`008Goo00<007ooOol07goo00<007ooOol0EGoo00<0
07ooOol0PWoo000QOol00`00Oomoo`0OOol00`00Oomoo`1DOol00`00Oomoo`23Ool0025oo`80025o
o`03001oogoo059oo`03001oogoo08Aoo`008Goo00<007ooOol08Goo00<007ooOol0DGoo00<007oo
Ool0Q7oo000QOol00`00Oomoo`0ROol00`00Oomoo`1?Ool00`00Oomoo`25Ool0025oo`03001oogoo
02=oo`03001oogoo04eoo`03001oogoo08Ioo`008Goo0P009Goo00<007ooOol0Bgoo00<007ooOol0
Qgoo000QOol00`00Oomoo`0TOol00`00Oomoo`1:Ool00`00Oomoo`28Ool000aoo`03001oogoo009o
o`03001oogoo009oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo02Eoo`03001oogoo04Qo
o`03001oogoo08Uoo`002goo00<007oo000027oo00L007ooOomoo`00Ool00004Ool00`00Oomoo`0V
Ool00`00Oomoo`17Ool00`00Oomoo`29Ool000]oo`03001oo`0000Qoo`03001oogoo00=oo`03001o
ogoo009oo`8002Qoo`03001oogoo04Eoo`03001oogoo08Yoo`002goo00<007oo00002Goo00<007oo
Ool00P001Goo00<007ooOol0:7oo00<007ooOol0@goo00<007ooOol0Rgoo000;Ool00`00Ool00007
Ool00`00Ool00002Ool00`00Oomoo`04Ool00`00Oomoo`0YOol00`00Oomoo`11Ool00`00Oomoo`2<
Ool000aoo`03001oogoo00Ioo`<0009oo`<000Aoo`03001oogoo02Uoo`03001oogoo03moo`8008mo
o`008Goo0P00:goo00<007ooOol0?Goo00<007ooOol0Sgoo000QOol00`00Oomoo`0[Ool00`00Oomo
o`0kOol00`00Oomoo`2@Ool0025oo`03001oogoo02aoo`03001oogoo03Uoo`03001oogoo095oo`00
8Goo00<007ooOol0;Goo0P00>7oo00<007ooOol0TWoo000QOol2000`Ool00`00Oomoo`0cOol2002E
Ool0025oo`03001oogoo031oo`03001oogoo035oo`03001oogoo09Eoo`008Goo00<007ooOol0<Goo
0P00<7oo00<007ooOol0UWoo000QOol00`00Oomoo`0cOol00`00Oomoo`0/Ool00`00Oomoo`2GOol0
025oo`8003Eoo`8002Yoo`8009Yoo`008Goo00<007ooOol0=Woo00<007ooOol09Woo00<007ooOol0
VWoo000QOol00`00Oomoo`0gOol2000UOol00`00Oomoo`2KOol0025oo`03001oogoo03Uoo`80025o
o`8009ioo`008Goo0P00?7oo0P007Goo0P00X7oo000QOol00`00Oomoo`0mOol2000IOol2002ROol0
01Uoo`D000=oo`03001oogoo03moo`8001Eoo`800:Aoo`006goo00<007ooOol00goo00<007ooOol0
@Goo0P0047oo0`00YWoo000KOol00`00Oomoo`03Ool20014Ool@002YOol001]oo`03001oogoo0?mo
o`=oo`006goo00<007ooOol0ogoo0goo000JOol2003oOol5Ool00?moob5oo`00ogoo8Goo003oOolQ
Ool00?moob5oo`00ogoo8Goo0000\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.671564, -1.10139, 
  0.0201972, 0.0124494}},
  CellTags->"BasicOperation"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      StyleBox["EvaluateExpression",
        FontFamily->"Courier",
        FontWeight->"Bold"], "[", 
      "\"\<plot([exp(x),ln(x)],x=0.1..1.5,title=`A plot`)\>\"", "]"}]], 
  "Input",
  CellTags->"BasicOperation"],

Cell[GraphicsData["PostScript", "\<\

%!

%%Creator: Mathematica

%%AspectRatio: .61803 

MathPictureStart

/Mabs {

Mgmatrix idtransform

Mtmatrix dtransform

} bind def

/Mabsadd { Mabs

3 -1 roll add

3 1 roll add

exch } bind def

%% Graphics

%%IncludeResource: font Courier

%%IncludeFont: Courier

/Courier findfont 10  scalefont  setfont

% Scaling calculations

0.0238095 0.634921 0.214487 0.08676 [

[.15079 .20199 -9 -9 ]

[.15079 .20199 9 0 ]

[.27778 .20199 -9 -9 ]

[.27778 .20199 9 0 ]

[.40476 .20199 -9 -9 ]

[.40476 .20199 9 0 ]

[.53175 .20199 -9 -9 ]

[.53175 .20199 9 0 ]

[.65873 .20199 -3 -9 ]

[.65873 .20199 3 0 ]

[.78571 .20199 -9 -9 ]

[.78571 .20199 9 0 ]

[.9127 .20199 -9 -9 ]

[.9127 .20199 9 0 ]

[.01131 .04097 -12 -4.5 ]

[.01131 .04097 0 4.5 ]

[.01131 .12773 -12 -4.5 ]

[.01131 .12773 0 4.5 ]

[.01131 .30125 -6 -4.5 ]

[.01131 .30125 0 4.5 ]

[.01131 .38801 -6 -4.5 ]

[.01131 .38801 0 4.5 ]

[.01131 .47477 -6 -4.5 ]

[.01131 .47477 0 4.5 ]

[.01131 .56153 -6 -4.5 ]

[.01131 .56153 0 4.5 ]

[.5 .63053 -20 0 ]

[.5 .63053 20 12.5625 ]

[ 0 0 0 0 ]

[ 1 .61803 0 0 ]

] MathScale

% Start of Graphics

1 setlinecap

1 setlinejoin

newpath

0 g

.25 Mabswid

.15079 .21449 m

.15079 .22074 L

s

[(0.2)] .15079 .20199 0 1 Mshowa

.27778 .21449 m

.27778 .22074 L

s

[(0.4)] .27778 .20199 0 1 Mshowa

.40476 .21449 m

.40476 .22074 L

s

[(0.6)] .40476 .20199 0 1 Mshowa

.53175 .21449 m

.53175 .22074 L

s

[(0.8)] .53175 .20199 0 1 Mshowa

.65873 .21449 m

.65873 .22074 L

s

[(1)] .65873 .20199 0 1 Mshowa

.78571 .21449 m

.78571 .22074 L

s

[(1.2)] .78571 .20199 0 1 Mshowa

.9127 .21449 m

.9127 .22074 L

s

[(1.4)] .9127 .20199 0 1 Mshowa

.125 Mabswid

.05556 .21449 m

.05556 .21824 L

s

.0873 .21449 m

.0873 .21824 L

s

.11905 .21449 m

.11905 .21824 L

s

.18254 .21449 m

.18254 .21824 L

s

.21429 .21449 m

.21429 .21824 L

s

.24603 .21449 m

.24603 .21824 L

s

.30952 .21449 m

.30952 .21824 L

s

.34127 .21449 m

.34127 .21824 L

s

.37302 .21449 m

.37302 .21824 L

s

.43651 .21449 m

.43651 .21824 L

s

.46825 .21449 m

.46825 .21824 L

s

.5 .21449 m

.5 .21824 L

s

.56349 .21449 m

.56349 .21824 L

s

.59524 .21449 m

.59524 .21824 L

s

.62698 .21449 m

.62698 .21824 L

s

.69048 .21449 m

.69048 .21824 L

s

.72222 .21449 m

.72222 .21824 L

s

.75397 .21449 m

.75397 .21824 L

s

.81746 .21449 m

.81746 .21824 L

s

.84921 .21449 m

.84921 .21824 L

s

.88095 .21449 m

.88095 .21824 L

s

.94444 .21449 m

.94444 .21824 L

s

.97619 .21449 m

.97619 .21824 L

s

.25 Mabswid

0 .21449 m

1 .21449 L

s

.02381 .04097 m

.03006 .04097 L

s

[(-2)] .01131 .04097 1 0 Mshowa

.02381 .12773 m

.03006 .12773 L

s

[(-1)] .01131 .12773 1 0 Mshowa

.02381 .30125 m

.03006 .30125 L

s

[(1)] .01131 .30125 1 0 Mshowa

.02381 .38801 m

.03006 .38801 L

s

[(2)] .01131 .38801 1 0 Mshowa

.02381 .47477 m

.03006 .47477 L

s

[(3)] .01131 .47477 1 0 Mshowa

.02381 .56153 m

.03006 .56153 L

s

[(4)] .01131 .56153 1 0 Mshowa

.125 Mabswid

.02381 .05832 m

.02756 .05832 L

s

.02381 .07567 m

.02756 .07567 L

s

.02381 .09302 m

.02756 .09302 L

s

.02381 .11038 m

.02756 .11038 L

s

.02381 .14508 m

.02756 .14508 L

s

.02381 .16243 m

.02756 .16243 L

s

.02381 .17978 m

.02756 .17978 L

s

.02381 .19714 m

.02756 .19714 L

s

.02381 .23184 m

.02756 .23184 L

s

.02381 .24919 m

.02756 .24919 L

s

.02381 .26654 m

.02756 .26654 L

s

.02381 .2839 m

.02756 .2839 L

s

.02381 .3186 m

.02756 .3186 L

s

.02381 .33595 m

.02756 .33595 L

s

.02381 .3533 m

.02756 .3533 L

s

.02381 .37066 m

.02756 .37066 L

s

.02381 .40536 m

.02756 .40536 L

s

.02381 .42271 m

.02756 .42271 L

s

.02381 .44006 m

.02756 .44006 L

s

.02381 .45742 m

.02756 .45742 L

s

.02381 .49212 m

.02756 .49212 L

s

.02381 .50947 m

.02756 .50947 L

s

.02381 .52682 m

.02756 .52682 L

s

.02381 .54418 m

.02756 .54418 L

s

.02381 .02362 m

.02756 .02362 L

s

.02381 .00626 m

.02756 .00626 L

s

.02381 .57888 m

.02756 .57888 L

s

.02381 .59623 m

.02756 .59623 L

s

.02381 .61358 m

.02756 .61358 L

s

.25 Mabswid

.02381 0 m

.02381 .61803 L

s

gsave

.5 .63053 -81 -4 Mabsadd m

1 1 Mabs scale

currentpoint translate

0 20.5625 translate 1 -1 scale

gsave

0.000000 0.000000 0.000000 setrgbcolor

1.000000 setlinewidth

gsave

newpath

 61.000000 16.562500 moveto

 460.000000 16.562500 lineto

 460.000000 4.000000 lineto

 61.000000 4.000000 lineto

 61.000000 16.562500 lineto

closepath

clip newpath

63.000000 12.812500 moveto

%%IncludeResource: font Courier

%%IncludeFont: Courier

/Courier findfont 10.000000 scalefont

[1 0 0 -1 0 0 ] makefont setfont

0.000000 0.000000 0.000000 setrgbcolor

0.000000 0.000000 rmoveto

63.000000 12.812500 moveto

%%IncludeResource: font Courier

%%IncludeFont: Courier

/Courier findfont 10.000000 scalefont

[1 0 0 -1 0 0 ] makefont setfont

0.000000 0.000000 0.000000 setrgbcolor

(A) show

75.000000 12.812500 moveto

(plot) show

99.000000 12.812500 moveto

%%IncludeResource: font Courier

%%IncludeFont: Courier

/Courier findfont 10.000000 scalefont

[1 0 0 -1 0 0 ] makefont setfont

0.000000 0.000000 0.000000 setrgbcolor

0.000000 0.000000 rmoveto

1.000000 setlinewidth

grestore

grestore

%%DocumentNeededResources: font Courier

%%DocumentSuppliedResources:

%%DocumentNeededFonts: Courier

%%DocumentSuppliedFonts:

%%DocumentFonts: font Courier

grestore

0 0 m

1 0 L

1 .61803 L

0 .61803 L

closepath

clip

newpath

.5 Mabswid

.0873 .31037 m

.12336 .31598 L

.16269 .32246 L

.19962 .32893 L

.23514 .33551 L

.27295 .34294 L

.30934 .35052 L

.34802 .35906 L

.38529 .3678 L

.42115 .37671 L

.45929 .38675 L

.49601 .39701 L

.53133 .40745 L

.56893 .41922 L

.60512 .43123 L

.64359 .44477 L

.68065 .45861 L

.7163 .47271 L

.75424 .48861 L

.79076 .50484 L

.82957 .52314 L

.86696 .54186 L

.90294 .56095 L

.94121 .58248 L

.97619 .60332 L

s

.0873 .01472 m

.10486 .03589 L

.12336 .05374 L

.16269 .08262 L

.19821 .10238 L

.23601 .1194 L

.27241 .13314 L

.31109 .14568 L

.34836 .15627 L

.38421 .16536 L

.42235 .17408 L

.45908 .18173 L

.49439 .1885 L

.532 .19517 L

.56818 .20114 L

.60666 .20706 L

.64372 .21241 L

.67937 .21726 L

.7173 .22214 L

.75382 .2266 L

.79263 .23109 L

.83003 .23521 L

.86601 .239 L

.90428 .24285 L

.94113 .24641 L

.97619 .24967 L

s

% End of Graphics

MathPictureEnd


\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40O003h00O5goo00<007ooOol0
ogoo1goo000GOol2003oOol8Ool001Moo`03001oogoo0?moo`Moo`005goo00<007ooOol03Woo00<0
07ooOol0mGoo000GOol00`00Oomoo`0?Ool00`00Oomoo`3dOol001Moo`03001oogoo011oo`03001o
ogoo0?=oo`005goo0P004Woo00<007ooOol0lWoo000GOol00`00Oomoo`0AOol00`00Oomoo`3bOol0
011oo`<000Aoo`03001oogoo019oo`03001oogoo0?5oo`0047oo00<007ooOol017oo00<007ooOol0
4goo00<007ooOol0l7oo000;Ool30003Ool00`00Oomoo`03Ool3000DOol00`00Oomoo`3_Ool0015o
o`03001oogoo00=oo`03001oogoo01Eoo`03001oogoo0>ioo`0047oo00<007oo000017oo00<007oo
Ool05Goo00<007ooOol0kWoo000AOol00`00Oomoo`03Ool00`00Oomoo`0FOol00`00Oomoo`3]Ool0
01Moo`03001oogoo01Moo`03001oogoo0>aoo`005goo0P006Goo0P00k7oo000GOol00`00Oomoo`0J
Ool00`00Oomoo`3YOol001Moo`03001oogoo01]oo`800>Uoo`005goo00<007ooOol07Goo0P00igoo
000GOol2000POol00`00Oomoo`3TOol001Moo`03001oogoo021oo`800>Aoo`005goo00<007ooOol0
8Woo00<007ooOol0hGoo000GOol00`00Oomoo`0SOol2003QOol001Moo`03001oogoo02Eoo`800=mo
o`005goo0P00:7oo0P00gGoo000GOol00`00Oomoo`0YOol2003KOol001Moo`03001oogoo02]oo`03
001oogoo0=Qoo`005goo00<007ooOol0;7oo0P00f7oo000GOol2000_Ool2003FOol001Moo`03001o
ogoo031oo`800=Aoo`005goo00<007ooOol0<Woo0P00dWoo000?Ool50003Ool00`00Oomoo`0dOol3
003?Ool0015oo`03001oogoo00=oo`03001oogoo03Moo`<00<aoo`002goo0`000goo00<007ooOol0
0goo0`00>Woo0`00bGoo000AOol00`00Oomoo`03Ool00`00Oomoo`0mOol30036Ool0015oo`03001o
ogoo00=oo`03001oogoo041oo`800<Aoo`0047oo0P001Goo00<007ooOol0@Woo0`00`Goo000GOol2
0016Ool2002oOol001Moo`03001oogoo04Moo`@00;]oo`005goo00<007ooOol0Bgoo1@00]Woo000G
Ool00`00Oomoo`1@Ool4002bOol001Moo`03001oogoo05Aoo`<00:moo`005goo0P00F7oo0`00[7oo
000GOol00`00Oomoo`1JOol4002XOol001Moo`03001oogoo01Uoo`03001oogoo009oo`04001oogoo
Ool3000EOol00`00Oomoo`02Ool00`00Oomoo`02Ool3000DOol00`00Oomoo`0500000goo0000000F
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`0GOol5000HOol50002Ool01000Oomoogoo
0`004goo1@000Woo00<007ooOol00Woo0`006Goo000GOol00`00Oomoo`0HOol00`00Ool00007Ool0
0`00Oomoo`0DOol00`00Ool00009Ool00`00Oomoo`0BOol00`00Ool00006Ool5000COol00`00Ool0
0007Ool00`00Ool0000JOol00`00Oomoo`0JOol00`00Oomoo`06Ool00`00Oomoo`0EOol00`00Oomo
o`08Ool00`00Oomoo`0HOol001Moo`8001Uoo`03001oo`0000Qoo`03001oogoo01=oo`03001oo`00
00Moo`@001=oo`03001oo`0000Moo`04001oo`00Ool5000>Ool00`00Ool00007Ool00`00Ool0000J
Ool00`00Oomoo`0JOol00`00Oomoo`07Ool00`00Oomoo`0DOol00`00Oomoo`06Ool4000IOol001Mo
o`03001oogoo01Qoo`03001oo`0000Qoo`03001oogoo01=oo`03001oo`0000Moo`03001oo`0001Ao
o`03001oo`0000Moo`8000Moo`D000Uoo`03001oo`0000Qoo`03001oogoo01Uoo`03001oogoo01Yo
o`03001oogoo00Moo`03001oogoo01Aoo`03001oogoo00Ioo`03001oo`0001Yoo`005goo00<007oo
Ool067oo00<007oo00001goo00<007oo000057oo00<007oo000027oo0P0057oo00<007oo00001goo
00<007ooOol02goo10001Goo00<007oo00001goo00<007oo00006Woo00<007ooOol06Woo00<007oo
Ool01Woo00<007oo00005Goo00<007ooOol01goo0P006Woo000GOol00`00Oomoo`0IOol00`00Oomo
o`07Ool00`00Oomoo`0DOol00`00Oomoo`08Ool00`00Oomoo`0COol00`00Oomoo`07Ool2000?Ool7
0009Ool00`00Oomoo`0HOol2000KOol20009Ool00`00Oomoo`0COol2000:Ool00`00Oomoo`0HOol0
01Moo`03001oogoo07eoo`P0085oo`005goo0P00QWoo1000OGoo000GOol00`00Oomoo`29Ool5001h
Ool001Moo`03001oogoo08ioo`D007=oo`005goo00<007ooOol0Tgoo1`00K7oo000AOooo00070009
Ool001Moo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03001o
ogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo00Eoo`03001o
ogoo00Eoo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001o
ogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Ioo`03001o
ogoo00Eoo`X000Ioo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Io
o`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo00eo
o`005goo00<007ooOol07Woo00<007ooOol07goo00<007ooOol07Woo00<007ooOol07Woo00<007oo
Ool07Woo00<007ooOol01goo20003goo00<007ooOol07Woo00<007ooOol07Woo000GOol00`00Oomo
o`2eOol5001<Ool001Moo`03001oogoo0;Yoo`L004Eoo`005goo0P00`Woo2P00>goo000GOol00`00
Oomoo`3;Ool:000aOol001Moo`03001oogoo0=Eoo`T002Qoo`005goo00<007ooOol0gWoo2P007Woo
000GOol00`00Oomoo`3XOol:000DOol001Moo`800?=oo`D000moo`005goo00<007ooOol0ogoo1goo
000GOol00`00Oomoo`3oOol7Ool001Moo`03001oogoo0?moo`Moo`005goo0P00ogoo27oo000GOol0
0`00Oomoo`3oOol7Ool001Moo`03001oogoo0?moo`Moo`005goo00<007ooOol0ogoo1goo000GOol0
0`00Oomoo`3oOol7Ool001Moo`800?moo`Qoo`005goo00<007ooOol0ogoo1goo000?Ool50003Ool0
0`00Oomoo`3oOol7Ool0015oo`03001oogoo00=oo`03001oogoo0?moo`Moo`004Goo00<007ooOol0
0goo0`00ogoo1goo000AOol00`00Oomoo`03Ool00`00Oomoo`3oOol7Ool0015oo`03001oogoo00=o
o`03001oogoo00ioo`<00?Eoo`0047oo0P001Goo00<007ooOol04Goo1000lGoo000GOol00`00Oomo
o`0EOol5003/Ool001Moo`8001]oo`H00>Ioo`005goo00<007ooOol087oo1`00ggoo000GOol00`00
Oomoo`0WOol7003HOol001Moo`03001oogoo02ioo`@00=Aoo`005goo0P00<goo1@00cgoo000GOol0
0`00Oomoo`0gOol5003:Ool001Moo`03001oogoo03aoo`D00<Eoo`005goo00<007ooOol0@Goo1@00
`7oo000GOol00`00Oomoo`16Ool5002kOol001Moo`8004aoo`D00;Ioo`005goo00<007ooOol0D7oo
1@00/Goo000GOol00`00Oomoo`1EOol5002/Ool001Moo`03001oogoo05Yoo`@00:Qoo`005goo0P00
Ggoo0`00YGoo000GOol00`00Oomoo`1QOol3002ROol001Moo`03001oogoo06Aoo`@009ioo`0047oo
0`0017oo00<007ooOol0J7oo1@00VGoo000@Ool00`00Oomoo`04Ool00`00Oomoo`1]Ool4002EOol0
015oo`03001oogoo00=oo`<0075oo`<0099oo`004Goo00<007ooOol00goo00<007ooOol0M7oo0`00
Sgoo000@Ool00`00Ool00004Ool00`00Oomoo`1gOol3002<Ool0015oo`03001oogoo00=oo`03001o
ogoo07Yoo`<008Uoo`005goo0P00OWoo1000QGoo000GOol00`00Oomoo`21Ool30022Ool001Moo`03
001oogoo08Aoo`<007moo`005goo00<007ooOol0Qgoo0`00O7oo000GOol00`00Oomoo`2:Ool3001i
Ool001Moo`8008ioo`<007Ioo`005goo00<007ooOol0T7oo1000LWoo000GOol00`00Oomoo`2DOol3
001_Ool001Moo`03001oogoo09Moo`8006eoo`005goo0P00VWoo0`00JWoo000GOol00`00Oomoo`2L
Ool2001XOol001Moo`03001oogoo09ioo`<006Eoo`005goo00<007ooOol0XGoo0`00HWoo000GOol0
0`00Oomoo`2TOol4001NOol001Moo`800:Uoo`<005]oo`005goo00<007ooOol0Zgoo0P00FGoo000@
Ool20005Ool00`00Oomoo`2]Ool2001GOol0019oo`03001oogoo009oo`03001oogoo0:moo`8005Eo
o`004Goo00<007ooOol00goo0`00/Goo0`00DWoo000BOol00`00Oomoo`02Ool00`00Oomoo`2dOol2
001@Ool0011oo`03001oo`0000Aoo`03001oogoo0;Ioo`<004eoo`004Goo00<007ooOol00goo00<0
07ooOol0^Goo0P00Bgoo000GOol00`00Oomoo`2kOol30018Ool001Moo`800;moo`8004Ioo`005goo
00<007ooOol0`7oo0P00A7oo000GOol00`00Oomoo`32Ool20012Ool001Moo`03001oogoo0<Aoo`80
041oo`005goo0P00agoo0P00?Woo000GOol00`00Oomoo`38Ool2000lOol001Moo`03001oogoo0<Yo
o`8003Yoo`005goo00<007ooOol0c7oo0P00>7oo000GOol00`00Oomoo`3>Ool2000fOol001Moo`80
0=5oo`8003Aoo`005goo00<007ooOol0dWoo0P00<Woo000GOol00`00Oomoo`3DOol2000`Ool001Mo
o`03001oogoo0=Ioo`8002ioo`005goo0P00fGoo0P00;7oo000GOol00`00Oomoo`3JOol2000ZOol0
01Moo`03001oogoo0=aoo`8002Qoo`004Goo0`000goo00<007ooOol0gWoo0P009Woo000BOol00`00
Oomoo`02Ool00`00Oomoo`3POol2000TOol0011oo`@000=oo`<00>9oo`80029oo`0047oo00<007oo
000017oo00<007ooOol0i7oo0P0087oo000AOol20004Ool00`00Oomoo`3VOol2000NOol0019oo`03
001oogoo009oo`03001oogoo0>Qoo`8001aoo`005goo0P00jgoo0P006Woo000GOol00`00Oomoo`3/
Ool2000HOol001Moo`03001oogoo0>ioo`8001Ioo`005goo00<007ooOol0l7oo00<007ooOol04goo
000GOol00`00Oomoo`3aOol2000COol001Moo`800?Aoo`03001oogoo011oo`005goo00<007ooOol0
m7oo0P0047oo000GOol00`00Oomoo`3fOol00`00Oomoo`0=Ool001Moo`03001oogoo0?moo`Moo`00
5goo00<007ooOol0ogoo1goo000GOol2003oOol8Ool001Moo`03001oogoo0?moo`Moo`00ogoo8Goo
003oOolQOol00?moob5oo`00ogoo8Goo003oOolQOol008ioo`<008moo`00Sgoo00<007ooOol0SWoo
0022Ool200000goo00000008Ool30002Ool30003Ool20003Ool2001oOol0089oo`05001oogooOol0
0008Ool01000Oomoo`000Woo00D007ooOomoo`00009oo`03001oo`00009oo`03001oogoo07aoo`00
Pgoo0`002Goo00@007ooOol0009oo`05001oogooOol00002Ool00`00Ool00021Ool008=oo`03001o
o`0000Qoo`@000=oo`03001oogoo009oo`800003Ool00000008007moo`00Q7oo00<007ooOol03Woo
00<007ooOol01Woo00<007ooOol0Ogoo0023Ool2000?Ool2002:Ool00?moob5oo`00ogoo8Goo0000
\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-0.144319, -2.47222, 
  0.00604164, 0.0442135}},
  CellTags->"BasicOperation"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      StyleBox["EvaluateExpression",
        FontFamily->"Courier",
        FontWeight->"Bold"], "[", "\"\<cos(subs(z=x+y,z^2+3))\>\"", "]"}]], 
  "Input",
  CellTags->"BasicOperation"],

Cell[BoxData[
    \(Cos[3 + \((x + y)\)\^2]\)], "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      StyleBox["EvaluateExpression",
        FontFamily->"Courier",
        FontWeight->"Bold"], "[", "\"\<expand((x+2)^2*(y+2))\>\"", "]"}]], 
  "Input",
  CellTags->"BasicOperation"],

Cell[BoxData[
    \(8 + 8\ x + 2\ x\^2 + 4\ y + 4\ x\ y + x\^2\ y\)], "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{
      StyleBox["EvaluateExpression",
        FontFamily->"Courier",
        FontWeight->"Bold"], "[", "\"\<plot([cos(z),sin(z),z=0..2*Pi])\>\"", 
      "]"}]], "Input",
  CellTags->"BasicOperation"],

Cell[GraphicsData["PostScript", "\<\

%!

%%Creator: Mathematica

%%AspectRatio: .61803 

MathPictureStart

/Mabs {

Mgmatrix idtransform

Mtmatrix dtransform

} bind def

/Mabsadd { Mabs

3 -1 roll add

3 1 roll add

exch } bind def

%% Graphics

%%IncludeResource: font Courier

%%IncludeFont: Courier

/Courier findfont 10  scalefont  setfont

% Scaling calculations

0.499998 0.476192 0.309017 0.294302 [

[.02381 .29652 -6 -9 ]

[.02381 .29652 6 0 ]

[.2619 .29652 -12 -9 ]

[.2619 .29652 12 0 ]

[.73809 .29652 -9 -9 ]

[.73809 .29652 9 0 ]

[.97619 .29652 -3 -9 ]

[.97619 .29652 3 0 ]

[.4875 .01471 -12 -4.5 ]

[.4875 .01471 0 4.5 ]

[.4875 .16187 -24 -4.5 ]

[.4875 .16187 0 4.5 ]

[.4875 .45617 -18 -4.5 ]

[.4875 .45617 0 4.5 ]

[.4875 .60332 -6 -4.5 ]

[.4875 .60332 0 4.5 ]

[ 0 0 0 0 ]

[ 1 .61803 0 0 ]

] MathScale

% Start of Graphics

1 setlinecap

1 setlinejoin

newpath

0 g

.25 Mabswid

.02381 .30902 m

.02381 .31527 L

s

[(-1)] .02381 .29652 0 1 Mshowa

.2619 .30902 m

.2619 .31527 L

s

[(-0.5)] .2619 .29652 0 1 Mshowa

.73809 .30902 m

.73809 .31527 L

s

[(0.5)] .73809 .29652 0 1 Mshowa

.97619 .30902 m

.97619 .31527 L

s

[(1)] .97619 .29652 0 1 Mshowa

.125 Mabswid

.07143 .30902 m

.07143 .31277 L

s

.11904 .30902 m

.11904 .31277 L

s

.16666 .30902 m

.16666 .31277 L

s

.21428 .30902 m

.21428 .31277 L

s

.30952 .30902 m

.30952 .31277 L

s

.35714 .30902 m

.35714 .31277 L

s

.40476 .30902 m

.40476 .31277 L

s

.45238 .30902 m

.45238 .31277 L

s

.54762 .30902 m

.54762 .31277 L

s

.59524 .30902 m

.59524 .31277 L

s

.64286 .30902 m

.64286 .31277 L

s

.69048 .30902 m

.69048 .31277 L

s

.78571 .30902 m

.78571 .31277 L

s

.83333 .30902 m

.83333 .31277 L

s

.88095 .30902 m

.88095 .31277 L

s

.92857 .30902 m

.92857 .31277 L

s

.25 Mabswid

0 .30902 m

1 .30902 L

s

.5 .01471 m

.50625 .01471 L

s

[(-1)] .4875 .01471 1 0 Mshowa

.5 .16187 m

.50625 .16187 L

s

[(-0.5)] .4875 .16187 1 0 Mshowa

.5 .45617 m

.50625 .45617 L

s

[(0.5)] .4875 .45617 1 0 Mshowa

.5 .60332 m

.50625 .60332 L

s

[(1)] .4875 .60332 1 0 Mshowa

.125 Mabswid

.5 .04415 m

.50375 .04415 L

s

.5 .07358 m

.50375 .07358 L

s

.5 .10301 m

.50375 .10301 L

s

.5 .13244 m

.50375 .13244 L

s

.5 .1913 m

.50375 .1913 L

s

.5 .22073 m

.50375 .22073 L

s

.5 .25016 m

.50375 .25016 L

s

.5 .27959 m

.50375 .27959 L

s

.5 .33845 m

.50375 .33845 L

s

.5 .36788 m

.50375 .36788 L

s

.5 .39731 m

.50375 .39731 L

s

.5 .42674 m

.50375 .42674 L

s

.5 .4856 m

.50375 .4856 L

s

.5 .51503 m

.50375 .51503 L

s

.5 .54446 m

.50375 .54446 L

s

.5 .57389 m

.50375 .57389 L

s

.25 Mabswid

.5 0 m

.5 .61803 L

s

0 0 m

1 0 L

1 .61803 L

0 .61803 L

closepath

clip

newpath

.5 Mabswid

.97619 .30902 m

.97618 .3113 L

.97614 .31338 L

.97606 .31577 L

.97597 .31804 L

.97572 .32207 L

.97536 .32643 L

.97483 .33124 L

.97414 .33633 L

.97253 .34544 L

.97039 .35481 L

.96795 .36354 L

.96081 .38322 L

.95129 .40294 L

.94065 .42058 L

.91017 .45852 L

.87673 .48902 L

.83352 .51908 L

.78673 .54399 L

.73155 .56618 L

.7028 .57529 L

.67066 .58377 L

.64214 .5899 L

.61017 .59533 L

.5949 .59742 L

.57842 .5993 L

.5641 .60064 L

.54845 .60179 L

.53926 .60232 L

.53074 .60271 L

.52259 .60299 L

.51847 .6031 L

.51392 .60319 L

.50983 .60326 L

.50609 .60329 L

.50197 .60332 L

.4999 .60332 L

.49764 .60332 L

.49381 .60329 L

.48973 .60325 L

.48587 .60319 L

.48236 .60312 L

.47512 .60292 L

.46715 .60262 L

.45109 .60176 L

.43626 .60067 L

.42244 .59939 L

.39155 .59559 L

.3589 .5901 L

.32884 .58365 L

Mistroke

.29695 .57522 L

.24178 .55629 L

.19207 .53351 L

.14489 .5051 L

.10369 .47218 L

.07215 .43822 L

.05955 .42088 L

.04825 .40209 L

.03956 .38409 L

.03328 .36743 L

.02828 .34924 L

.02631 .33917 L

.0256 .33453 L

.02497 .3296 L

.02454 .32539 L

.02436 .32315 L

.02418 .32074 L

.02406 .31853 L

.02396 .31651 L

.02389 .31458 L

.02384 .31254 L

.02381 .31032 L

.02381 .30795 L

.02384 .30571 L

.02388 .30368 L

.02397 .30129 L

.0241 .2987 L

.02443 .294 L

.02486 .28941 L

.02546 .2845 L

.02688 .27564 L

.02913 .26515 L

.03171 .25563 L

.03905 .23514 L

.04818 .21606 L

.07174 .18034 L

.10012 .14921 L

.13486 .1201 L

.17963 .09128 L

.22702 .06787 L

.28309 .04702 L

.3408 .03165 L

.37094 .02573 L

.38817 .02295 L

.40459 .02068 L

.41978 .01892 L

.43635 .01736 L

.45149 .01625 L

.4655 .01549 L

.47334 .01518 L

Mistroke

.48068 .01496 L

.48489 .01486 L

.48874 .0148 L

.49087 .01477 L

.49319 .01475 L

.49538 .01473 L

.49741 .01472 L

.49954 .01472 L

.50184 .01472 L

.50401 .01473 L

.50601 .01474 L

.5098 .01478 L

.5138 .01484 L

.5181 .01493 L

.52212 .01503 L

.53124 .01535 L

.5394 .01572 L

.54693 .01615 L

.56393 .01738 L

.57886 .01878 L

.59461 .02058 L

.62161 .02447 L

.65091 .02989 L

.70722 .04404 L

.76408 .06412 L

.81405 .08779 L

.86069 .11687 L

.89804 .14747 L

.92672 .1784 L

.94037 .19703 L

.95129 .2151 L

.95954 .23188 L

.9665 .24993 L

.9707 .26445 L

.97256 .27274 L

.97395 .28053 L

.97494 .2877 L

.97559 .29425 L

.97585 .29787 L

.97596 .29979 L

.97605 .30183 L

.97612 .30378 L

.97616 .30556 L

.97618 .30734 L

.97619 .30902 L

Mfstroke

% End of Graphics

MathPictureEnd


\
\>"], "Graphics",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40O003h00OSgoo00<007ooOol0
SWoo002?Ool00`00Oomoo`2>Ool008Ioo`D000Aoo`03001oogoo08ioo`00R7oo00<007ooOol017oo
00<007ooOol0SWoo0022OolM0021Ool007Moo`d000Aoo`03001oogoo00Aoo`03001oogoo00eoo`T0
07Qoo`00KGoo2P004Goo00<007ooOol017oo00<007ooOol05Woo1P00LWoo001YOol4000JOol20006
Ool00`00Oomoo`0LOol8001ZOol006Eoo`@002Ioo`03001oogoo02Aoo`H006Aoo`00H7oo1@00:Woo
00<007ooOol0:Woo1000H7oo001LOol4000_Ool00`00Oomoo`0^Ool4001LOol005Qoo`@003=oo`03
001oogoo039oo`@005Qoo`00E7oo1000=goo0P00=goo1000E7oo001@Ool4000kOol00`00Oomoo`0j
Ool2001BOol004ioo`8003moo`03001oogoo03aoo`<004moo`00Bgoo0`00@Goo00<007ooOol0?goo
0`00C7oo0018Ool30014Ool00`00Oomoo`12Ool2001:Ool004Ioo`8004Moo`03001oogoo04Aoo`<0
04Moo`00@goo0`00BGoo00<007ooOol0Agoo0`00A7oo0011Ool2001<Ool00`00Oomoo`1:Ool20012
Ool003moo`8004ioo`03001oogoo04aoo`80041oo`00?Goo0P00D7oo0P00Cgoo0P00?Woo000kOol2
001BOol00`00Oomoo`1@Ool2000lOol003Uoo`8005Aoo`03001oogoo059oo`8003Yoo`00=goo0P00
EWoo00<007ooOol0E7oo0P00>7oo000eOol2001HOol00`00Oomoo`1FOol2000fOol003Aoo`03001o
ogoo05Qoo`03001oogoo05Qoo`8003Aoo`00<Woo0P00Fgoo00<007ooOol0FWoo0P00<Woo000`Ool2
001MOol00`00Oomoo`1LOol00`00Oomoo`0_Ool002moo`03001oogoo05eoo`8005ioo`8002moo`00
;Goo0P00H7oo00<007ooOol0Ggoo00<007ooOol0;7oo000/Ool00`00Oomoo`1POol00`00Oomoo`1P
Ool2000/Ool002Yoo`8006=oo`03001oogoo069oo`8002Yoo`00:7oo0P00IGoo00<007ooOol0I7oo
00<007ooOol09goo000WOol00`00Oomoo`1UOol00`00Oomoo`1UOol00`00Oomoo`0VOol002Ioo`03
001oogoo06Ioo`03001oogoo06Ioo`8002Ioo`0097oo0P00JGoo00<007ooOol0J7oo00<007ooOol0
8goo000SOol00`00Oomoo`1YOol00`00Oomoo`1YOol00`00Oomoo`0ROol0029oo`03001oogoo06Yo
o`8006]oo`03001oogoo025oo`008Goo00<007ooOol0Jgoo00<007ooOol0Jgoo00<007ooOol087oo
000OOol2001^Ool00`00Oomoo`1/Ool2000POol001ioo`03001oogoo06ioo`03001oogoo06ioo`03
001oogoo01eoo`007Goo00<007ooOol0Kgoo00<007ooOol0Kgoo00<007ooOol077oo000LOol00`00
Oomoo`1`Ool00`00Oomoo`1`Ool00`00Oomoo`0KOol001]oo`03001oogoo061oo`03001oogoo009o
o`03001oogoo009oo`03001oogoo00Aoo`03001oogoo075oo`03001oogoo01Yoo`006Woo00<007oo
Ool0H7oo00<007oo00001goo00<007oo00001Goo00<007ooOol0LWoo00<007ooOol06Goo000IOol0
0`00Oomoo`1LOol30002Ool00`00Ool00009Ool00`00Oomoo`03Ool3001cOol00`00Oomoo`0HOol0
01Uoo`03001oogoo065oo`03001oo`0000Moo`8000Ioo`03001oogoo07=oo`03001oogoo01Qoo`00
67oo00<007ooOol0HWoo00<007oo00001goo00<007ooOol01Goo00<007ooOol0M7oo00<007ooOol0
5goo000GOol00`00Oomoo`1TOol00`00Oomoo`06Ool30005Ool00`00Oomoo`1eOol00`00Oomoo`0F
Ool001Ioo`03001oogoo07Ioo`03001oogoo07Ioo`03001oogoo01Eoo`005Goo00<007ooOol0Mgoo
00<007ooOol0Mgoo00<007ooOol057oo000DOol00`00Oomoo`1hOol00`00Oomoo`1hOol00`00Oomo
o`0COol001Aoo`03001oogoo07Qoo`03001oogoo07Uoo`03001oogoo019oo`004goo00<007ooOol0
NGoo0P00NWoo00<007ooOol04Woo000BOol00`00Oomoo`1jOol00`00Oomoo`1jOol00`00Oomoo`0A
Ool0019oo`03001oogoo07Yoo`03001oogoo07]oo`03001oogoo011oo`004Goo00<007ooOol0Ngoo
00<007ooOol0O7oo00<007ooOol03goo000AOol00`00Oomoo`1kOol00`00Oomoo`1lOol00`00Oomo
o`0?Ool0011oo`03001oogoo07aoo`03001oogoo07eoo`03001oogoo00ioo`003goo00<007ooOol0
OGoo00<007ooOol0OGoo00<007ooOol03Woo000?Ool00`00Oomoo`1mOol00`00Oomoo`1nOol00`00
Oomoo`0=Ool000ioo`03001oogoo07ioo`03001oogoo07ioo`03001oogoo00eoo`003Goo00<007oo
Ool0Ogoo0P00P7oo00<007ooOol037oo000=Ool00`00Oomoo`1oOol00`00Oomoo`1oOol00`00Oomo
o`0<Ool000aoo`03001oogoo081oo`03001oogoo081oo`03001oogoo00]oo`0037oo00<007ooOol0
P7oo00<007ooOol0P7oo00<007ooOol02goo000;Ool00`00Oomoo`21Ool00`00Oomoo`20Ool00`00
Oomoo`0;Ool000]oo`03001oogoo085oo`03001oogoo085oo`03001oogoo00Yoo`002Woo00<007oo
Ool0PWoo00<007ooOol0PGoo00<007ooOol02Woo000:Ool00`00Oomoo`22Ool00`00Oomoo`22Ool0
0`00Oomoo`09Ool000Yoo`03001oogoo089oo`8008=oo`03001oogoo00Uoo`002Goo00<007ooOol0
Pgoo00<007ooOol0PWoo00<007ooOol02Goo0009Ool00`00Oomoo`23Ool00`00Oomoo`23Ool00`00
Oomoo`08Ool000Uoo`03001oogoo08=oo`03001oogoo08=oo`03001oogoo00Qoo`002Goo00<007oo
Ool0Pgoo00<007ooOol0Pgoo00<007ooOol027oo0007Ool5000lOol00`00Oomoo`02Ool00`00Oomo
o`02Ool00`00Oomoo`0jOol00`00Oomoo`0kOol00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomo
o`0jOol50007Ool000Uoo`03001oogoo03]oo`03001oo`0000Moo`03001oo`0003]oo`03001oogoo
03Yoo`03001oo`0000Moo`03001oo`0003eoo`03001oogoo00Moo`000goo0`000Woo0P00>7oo0`00
0Woo00<007oo00002Goo00<007ooOol0>Goo00<007ooOol0>Woo00<007oo00002Goo00<007ooOol0
>goo00<007ooOol01goo0008Ool2000mOol00`00Ool00007Ool2000lOol00`00Oomoo`0jOol00`00
Ool00007Ool2000nOol00`00Oomoo`07Ool000Qoo`8003eoo`03001oo`0000Moo`03001oogoo03]o
o`8003]oo`03001oo`0000Moo`03001oogoo03eoo`03001oogoo00Moo`0027oo0P00?Woo00<007oo
Ool01Woo0`00>goo00<007ooOol0>goo00<007ooOol01Woo0`00?7oo0`0027oo0007Ool00`00Oomo
o`25Ool00`00Oomoo`25Ool00`00Oomoo`06Ool000Moo`03001oogoo08Eoo`03001oogoo08Eoo`03
001oogoo00Ioo`001goo00<007ooOol0QGoo00<007ooOol0QGoo00<007ooOol01Woo0007Ool00`00
Oomoo`25Ool00`00Oomoo`25Ool00`00Oomoo`06Ool000Moo`03001oogoo08Eoo`03001oogoo08Eo
o`03001oogoo00Ioo`001goo00<007ooOol0QGoo00<007ooOol0QGoo00<007ooOol01Woo003o000P
0001Ool000Moo`03001oogoo00]oo`03001oogoo00Yoo`03001oogoo00]oo`03001oogoo00Yoo`03
001oogoo00]oo`03001oogoo00]oo`03001oogoo00Yoo`03001oogoo00]oo`03001oogoo00Yoo`03
001oogoo00]oo`03001oogoo00]oo`03001oogoo00Yoo`03001oogoo00]oo`03001oogoo00Yoo`03
001oogoo00]oo`03001oogoo00]oo`03001oogoo00Yoo`03001oogoo00]oo`03001oogoo00Yoo`03
001oogoo00]oo`03001oogoo00Ioo`001goo00<007ooOol0@Goo00<007ooOol0@Goo00<007ooOol0
@Goo00<007ooOol0@Goo00<007ooOol01Woo0007Ool00`00Oomoo`25Ool00`00Oomoo`25Ool00`00
Oomoo`06Ool000Moo`03001oogoo08Eoo`03001oogoo08Eoo`03001oogoo00Ioo`001goo00<007oo
Ool0QGoo00<007ooOol0QGoo00<007ooOol01Woo0007Ool00`00Oomoo`25Ool00`00Oomoo`24Ool0
0`00Oomoo`07Ool000Qoo`03001oogoo08Aoo`03001oogoo08Aoo`03001oogoo00Moo`0027oo00<0
07ooOol0Q7oo0P00QGoo00<007ooOol01goo0008Ool00`00Oomoo`24Ool00`00Oomoo`24Ool00`00
Oomoo`07Ool000Qoo`03001oogoo08Aoo`03001oogoo08Aoo`03001oogoo00Moo`0027oo00<007oo
Ool0Q7oo00<007ooOol0Q7oo00<007ooOol01goo0008Ool00`00Oomoo`24Ool00`00Oomoo`23Ool0
0`00Oomoo`08Ool000Qoo`03001oogoo08Aoo`03001oogoo08=oo`03001oogoo00Qoo`002Goo00<0
07ooOol0Pgoo00<007ooOol0Pgoo00<007ooOol027oo0009Ool00`00Oomoo`23Ool00`00Oomoo`23
Ool00`00Oomoo`08Ool000Yoo`03001oogoo089oo`03001oogoo08=oo`03001oogoo00Qoo`002Woo
00<007ooOol0PWoo0P00Pgoo00<007ooOol02Goo000;Ool00`00Oomoo`21Ool00`00Oomoo`22Ool0
0`00Oomoo`09Ool000]oo`03001oogoo085oo`03001oogoo085oo`03001oogoo00Yoo`0037oo00<0
07ooOol0P7oo00<007ooOol0PGoo00<007ooOol02Woo000<Ool00`00Oomoo`20Ool00`00Oomoo`20
Ool00`00Oomoo`0;Ool000aoo`03001oogoo081oo`03001oogoo081oo`03001oogoo00]oo`003Goo
00<007ooOol0Ogoo00<007ooOol0Ogoo00<007ooOol037oo000=Ool00`00Oomoo`1oOol00`00Oomo
o`1oOol00`00Oomoo`0<Ool000eoo`03001oogoo07moo`80081oo`03001oogoo00aoo`003Woo00<0
07ooOol0OWoo00<007ooOol0OWoo00<007ooOol03Goo000>Ool00`00Oomoo`1nOol00`00Oomoo`1n
Ool00`00Oomoo`0=Ool000moo`03001oogoo07eoo`03001oogoo07eoo`03001oogoo00ioo`003goo
00<007ooOol0OGoo00<007ooOol0OGoo00<007ooOol03Woo000@Ool00`00Oomoo`1lOol00`00Oomo
o`1lOol00`00Oomoo`0?Ool0011oo`03001oogoo07aoo`03001oogoo07aoo`03001oogoo00moo`00
4Goo00<007ooOol0Ngoo00<007ooOol0Ngoo00<007ooOol047oo000BOol00`00Oomoo`1jOol00`00
Oomoo`1jOol00`00Oomoo`0AOol001=oo`03001oogoo07Uoo`8007Yoo`03001oogoo019oo`004goo
00<007ooOol0NGoo00<007ooOol0NGoo00<007ooOol04Woo000DOol00`00Oomoo`1hOol00`00Oomo
o`1hOol00`00Oomoo`0COol001Eoo`03001oogoo07Moo`03001oogoo07Moo`03001oogoo01Aoo`00
5Woo00<007ooOol0MWoo00<007ooOol0MWoo00<007ooOol05Goo000GOol00`00Oomoo`1eOol00`00
Oomoo`1eOol00`00Oomoo`0FOol001Qoo`03001oogoo06=oo`03001oogoo009oo`03001oogoo009o
o`03001oogoo00Aoo`03001oogoo07Aoo`03001oogoo01Moo`006Goo00<007ooOol0HGoo00<007oo
00001goo00<007oo00001Goo00<007ooOol0M7oo00<007ooOol05goo000JOol00`00Oomoo`1POol0
0`00Ool00009Ool00`00Oomoo`03Ool3001cOol00`00Oomoo`0HOol001Yoo`03001oogoo061oo`03
001oo`0000Moo`8000Ioo`03001oogoo079oo`03001oogoo01Uoo`006goo00<007ooOol0Ggoo00<0
07oo00001goo00<007ooOol01Goo00<007ooOol0LGoo00<007ooOol06Woo000LOol00`00Oomoo`1O
Ool00`00Oomoo`06Ool30005Ool00`00Oomoo`1_Ool2000MOol001eoo`03001oogoo06moo`03001o
ogoo06ioo`03001oogoo01eoo`007Woo00<007ooOol0KWoo00<007ooOol0KGoo00<007ooOol07Woo
000OOol00`00Oomoo`1]Ool00`00Oomoo`1/Ool00`00Oomoo`0OOol0021oo`8006eoo`03001oogoo
06Yoo`80029oo`008Woo00<007ooOol0JWoo0P00JWoo00<007ooOol08Woo000SOol00`00Oomoo`1Y
Ool00`00Oomoo`1XOol00`00Oomoo`0SOol002Aoo`8006Uoo`03001oogoo06Moo`03001oogoo02Ao
o`009Woo00<007ooOol0IWoo00<007ooOol0IGoo0P009goo000WOol00`00Oomoo`1UOol00`00Oomo
o`1TOol00`00Oomoo`0WOol002Qoo`8006Eoo`03001oogoo06=oo`03001oogoo02Qoo`00:Woo00<0
07ooOol0HWoo00<007ooOol0HGoo0P00:goo000[Ool2001ROol00`00Oomoo`1POol00`00Oomoo`0[
Ool002eoo`80061oo`03001oogoo05moo`03001oogoo02aoo`00;goo00<007ooOol0GGoo0P00GWoo
0P00;goo000`Ool2001MOol00`00Oomoo`1LOol00`00Oomoo`0_Ool0039oo`03001oogoo05Yoo`03
001oogoo05Yoo`80039oo`00<goo0P00FWoo00<007ooOol0F7oo0P00=7oo000eOol2001HOol00`00
Oomoo`1FOol2000fOol003Moo`03001oogoo05Eoo`03001oogoo05Aoo`8003Qoo`00>7oo0P00EGoo
00<007ooOol0DWoo0P00>Woo000jOol2001COol00`00Oomoo`1@Ool2000lOol003aoo`80055oo`80
04moo`8003ioo`00?Woo0P00Cgoo00<007ooOol0C7oo0P00@7oo0010Ool2001=Ool00`00Oomoo`19
Ool30012Ool0049oo`8004]oo`03001oogoo04Ioo`<004Eoo`00A7oo0`00B7oo00<007ooOol0A7oo
0P00B7oo0017Ool30015Ool00`00Oomoo`11Ool3001:Ool004Yoo`<0049oo`03001oogoo03ioo`<0
04eoo`00CGoo1000?Woo00<007ooOol0?7oo0P00D7oo001AOol3000kOol00`00Oomoo`0iOol3001B
Ool005Aoo`@003Moo`8003Ioo`@005Eoo`00F7oo1000<goo00<007ooOol0<Goo1000FGoo001LOol5
000^Ool00`00Oomoo`0]Ool4001MOol0065oo`@002Yoo`03001oogoo02Uoo`@0065oo`00IGoo1@00
9Goo00<007ooOol097oo1@00IGoo001ZOol4000QOol00`00Oomoo`0OOol5001ZOol006ioo`L0015o
o`D000Aoo`03001oogoo01Ioo`T006moo`00MGoo2P002Goo00<007ooOol017oo00<007ooOol03Woo
2000N7oo001oOolQ0020Ool008Qoo`03001oogoo00Aoo`03001oogoo08ioo`00R7oo00<007ooOol0
17oo00<007ooOol0SWoo0027Ool20006Ool00`00Oomoo`2>Ool008moo`03001oogoo08ioo`00\
\>"],
  ImageRangeCache->{{{0, 287}, {176.938, 0}} -> {-1.05622, -1.05362, 
  0.00736041, 0.0119094}},
  CellTags->"BasicOperation"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellTags->"BasicOperation"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["ConvertWorkSheet", "Subsection",
  CellTags->"BasicOperation"],

Cell[TextData[{
  StyleBox["The ConvertWorksheet button, located", "MR",
    FontFamily->"Times New Roman"],
  StyleBox[
  " on the ConverterPalette, opens a standard file dialog box that lets you \
select a .mws file,  converts the file into a ",
    FontFamily->"Times New Roman"],
  StyleBox["Mathematica",
    FontFamily->"Times New Roman",
    FontSlant->"Italic"],
  StyleBox[" notebook, and opens the notebook",
    FontFamily->"Times New Roman"],
  StyleBox[". ",
    FontFamily->"Courier"]
}], "Text",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  CellTags->"BasicOperation"],

Cell[TextData[{
  "Open the ",
  ButtonBox["ConverterPalette",
    ButtonData:>{
      FrontEnd`FileName[ {
        ParentDirectory[ 
          ParentDirectory[ ]]}, "ConverterPalette.nb", CharacterEncoding -> 
        "WindowsANSI"], None},
    ButtonStyle->"Hyperlink"],
  ", click the ",
  StyleBox["ConvertWorksheet", "MR"],
  " button, and select ",
  StyleBox["sample.mws", "MR"],
  "."
}], "Text",
  CellTags->"BasicOperation"],

Cell[TextData[{
  "Alternatively, you can use the command ",
  StyleBox["NotebookPut[Notebook[ConvertWorkSheet[\"", "MR"],
  StyleBox["filepath",
    FontSlant->"Italic"],
  StyleBox["\"]]]", "MR"],
  " with the full pathname to the worksheet."
}], "Text",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NotebookPut[
      Notebook[ConvertWorkSheet[
          "\<AddOns\\Applications\\MapleConvert\\Documentation\\English\\\
sample.mws\>"]]]\)], "Input",
  CellTags->"BasicOperation"],

Cell[BoxData[
    InterpretationBox[
      \(\[SkeletonIndicator]  NotebookObject  \[SkeletonIndicator]\),
      NotebookObject[ 
        FrontEndObject[ 
          LinkObject[ "FM35477728", 1, 1]], 22],
      Editable->False]], "Output",
  CellTags->"BasicOperation"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Microsoft Windows 3.0",
ScreenRectangle->{{0, 1024}, {0, 712}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Working",
WindowSize->{500, 350},
WindowMargins->{{0, Automatic}, {Automatic, 5}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
StyleDefinitions -> "Manual.nb"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "GeneralInformation"->{
    Cell[1731, 51, 93, 2, 105, "Title",
      CellTags->"GeneralInformation"],
    Cell[1827, 55, 137, 6, 84, "Subsubtitle",
      CellTags->"GeneralInformation"],
    Cell[1967, 63, 109, 2, 33, "Text",
      CellTags->"GeneralInformation"],
    Cell[2101, 69, 72, 1, 72, "Section",
      CellTags->"GeneralInformation"],
    Cell[2176, 72, 1015, 24, 217, "Text",
      CellTags->"GeneralInformation"]},
  "BasicOperation"->{
    Cell[3228, 101, 77, 1, 72, "Section",
      CellTags->"BasicOperation"],
    Cell[3308, 104, 482, 16, 52, "Text",
      CellTags->"BasicOperation"],
    Cell[3793, 122, 13396, 222, 101, 13275, 218, "GraphicsData", 
    "Bitmap", "Picture",
      CellTags->"BasicOperation"],
    Cell[17192, 346, 164, 4, 52, "Text",
      CellTags->"BasicOperation"],
    Cell[17359, 352, 91, 1, 33, "Commentary",
      CellTags->"BasicOperation"],
    Cell[17453, 355, 86, 2, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[17564, 361, 69, 1, 56, "Subsection",
      CellTags->"BasicOperation"],
    Cell[17636, 364, 268, 8, 52, "Text",
      CellTags->"BasicOperation"],
    Cell[17907, 374, 130, 3, 33, "Commentary",
      CellTags->"BasicOperation"],
    Cell[18040, 379, 101, 2, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[18144, 383, 78, 2, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[18259, 390, 70, 1, 56, "Subsection",
      CellTags->"BasicOperation"],
    Cell[18332, 393, 550, 14, 90, "Text",
      CellTags->"BasicOperation"],
    Cell[18907, 411, 63, 1, 49, "Subsubsection",
      CellTags->"BasicOperation"],
    Cell[18995, 416, 200, 5, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[19198, 423, 79, 2, 29, "Output",
      CellTags->"BasicOperation"],
    Cell[19314, 430, 223, 6, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[19540, 438, 68, 2, 29, "Output",
      CellTags->"BasicOperation"],
    Cell[19645, 445, 221, 6, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[19869, 453, 15581, 859, 186, 5559, 730, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"BasicOperation"],
    Cell[35453, 1314, 160, 4, 29, "Output",
      CellTags->"BasicOperation"],
    Cell[35650, 1323, 243, 7, 50, "Input",
      CellTags->"BasicOperation"],
    Cell[35896, 1332, 14283, 877, 186, 6394, 774, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"BasicOperation"],
    Cell[50182, 2211, 160, 4, 29, "Output",
      CellTags->"BasicOperation"],
    Cell[50379, 2220, 212, 6, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[50594, 2228, 87, 2, 29, "Output",
      CellTags->"BasicOperation"],
    Cell[50718, 2235, 211, 6, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[50932, 2243, 109, 2, 29, "Output",
      CellTags->"BasicOperation"],
    Cell[51078, 2250, 225, 6, 30, "Input",
      CellTags->"BasicOperation"],
    Cell[51306, 2258, 15175, 809, 186, 5267, 682, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"BasicOperation"],
    Cell[66484, 3069, 160, 4, 29, "Output",
      CellTags->"BasicOperation"],
    Cell[66705, 3080, 68, 1, 56, "Subsection",
      CellTags->"BasicOperation"],
    Cell[66776, 3083, 540, 15, 71, "Text",
      CellTags->"BasicOperation"],
    Cell[67341, 3102, 62, 1, 49, "Subsubsection",
      CellTags->"BasicOperation"],
    Cell[67406, 3105, 434, 15, 33, "Text",
      CellTags->"BasicOperation"],
    Cell[67843, 3122, 286, 8, 52, "Text",
      CellTags->"BasicOperation"],
    Cell[68154, 3134, 200, 5, 70, "Input",
      CellTags->"BasicOperation"],
    Cell[68357, 3141, 268, 7, 29, "Output",
      CellTags->"BasicOperation"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"GeneralInformation", 69455, 3176},
  {"BasicOperation", 69881, 3187}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 93, 2, 105, "Title",
  CellTags->"GeneralInformation"],
Cell[1827, 55, 137, 6, 84, "Subsubtitle",
  CellTags->"GeneralInformation"],
Cell[1967, 63, 109, 2, 33, "Text",
  CellTags->"GeneralInformation"],

Cell[CellGroupData[{
Cell[2101, 69, 72, 1, 72, "Section",
  CellTags->"GeneralInformation"],
Cell[2176, 72, 1015, 24, 217, "Text",
  CellTags->"GeneralInformation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3228, 101, 77, 1, 72, "Section",
  CellTags->"BasicOperation"],
Cell[3308, 104, 482, 16, 52, "Text",
  CellTags->"BasicOperation"],
Cell[3793, 122, 13396, 222, 101, 13275, 218, "GraphicsData", 
"Bitmap", "Picture",
  CellTags->"BasicOperation"],
Cell[17192, 346, 164, 4, 52, "Text",
  CellTags->"BasicOperation"],
Cell[17359, 352, 91, 1, 33, "Commentary",
  CellTags->"BasicOperation"],
Cell[17453, 355, 86, 2, 30, "Input",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{
Cell[17564, 361, 69, 1, 56, "Subsection",
  CellTags->"BasicOperation"],
Cell[17636, 364, 268, 8, 52, "Text",
  CellTags->"BasicOperation"],
Cell[17907, 374, 130, 3, 33, "Commentary",
  CellTags->"BasicOperation"],
Cell[18040, 379, 101, 2, 30, "Input",
  CellTags->"BasicOperation"],
Cell[18144, 383, 78, 2, 30, "Input",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18259, 390, 70, 1, 56, "Subsection",
  CellTags->"BasicOperation"],
Cell[18332, 393, 550, 14, 90, "Text",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{
Cell[18907, 411, 63, 1, 49, "Subsubsection",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{
Cell[18995, 416, 200, 5, 30, "Input",
  CellTags->"BasicOperation"],
Cell[19198, 423, 79, 2, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19314, 430, 223, 6, 30, "Input",
  CellTags->"BasicOperation"],
Cell[19540, 438, 68, 2, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19645, 445, 221, 6, 30, "Input",
  CellTags->"BasicOperation"],
Cell[19869, 453, 15581, 859, 186, 5559, 730, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->"BasicOperation"],
Cell[35453, 1314, 160, 4, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[35650, 1323, 243, 7, 50, "Input",
  CellTags->"BasicOperation"],
Cell[35896, 1332, 14283, 877, 186, 6394, 774, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->"BasicOperation"],
Cell[50182, 2211, 160, 4, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50379, 2220, 212, 6, 30, "Input",
  CellTags->"BasicOperation"],
Cell[50594, 2228, 87, 2, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50718, 2235, 211, 6, 30, "Input",
  CellTags->"BasicOperation"],
Cell[50932, 2243, 109, 2, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51078, 2250, 225, 6, 30, "Input",
  CellTags->"BasicOperation"],
Cell[51306, 2258, 15175, 809, 186, 5267, 682, "GraphicsData", 
"PostScript", "Graphics",
  CellTags->"BasicOperation"],
Cell[66484, 3069, 160, 4, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[66705, 3080, 68, 1, 56, "Subsection",
  CellTags->"BasicOperation"],
Cell[66776, 3083, 540, 15, 71, "Text",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{
Cell[67341, 3102, 62, 1, 49, "Subsubsection",
  CellTags->"BasicOperation"],
Cell[67406, 3105, 434, 15, 33, "Text",
  CellTags->"BasicOperation"],
Cell[67843, 3122, 286, 8, 52, "Text",
  CellTags->"BasicOperation"],

Cell[CellGroupData[{
Cell[68154, 3134, 200, 5, 70, "Input",
  CellTags->"BasicOperation"],
Cell[68357, 3141, 268, 7, 29, "Output",
  CellTags->"BasicOperation"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

