(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     48745,       2008]*)
(*NotebookOutlinePosition[     50454,       2058]*)
(*  CellTagsIndexPosition[     50282,       2049]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Style Definitions", "Subtitle"],

Cell["\<\
Modify the definitions below to change the default appearance of \
all cells in a given style.  Make modifications to any definition using \
commands in the Format menu.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Style Environment Names", "Section"],

Cell[StyleData[All, "Working"],
  PageWidth->WindowWidth,
  ShowCellBracket->False,
  CellLabelMargins->{{10, Inherited}, {Inherited, Inherited}},
  ScriptMinSize->9],

Cell[StyleData[All, "AllBrackets"],
  PageWidth->WindowWidth,
  ShowCellBracket->True,
  ScriptMinSize->9],

Cell[StyleData[All, "Presentation"],
  PageWidth->WindowWidth,
  ShowCellBracket->False,
  ShowGroupOpenCloseIcon->True,
  ScriptMinSize->9],

Cell[StyleData[All, "Printout"],
  PageWidth->PaperWidth,
  ShowCellBracket->False,
  CellLabelMargins->{{2, Inherited}, {Inherited, Inherited}},
  ScriptMinSize->5,
  PrivateFontOptions->{"FontType"->"Outline"}],

Cell[StyleData[All, "EnhancedPrintout"],
  PageWidth->PaperWidth,
  ShowCellBracket->False,
  CellLabelMargins->{{2, Inherited}, {Inherited, Inherited}},
  ScriptMinSize->5,
  PrivateFontOptions->{"FontType"->"Outline"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Notebook Options", "Section"],

Cell["\<\
The options defined for the style below will be used at the \
Notebook level.\
\>", "Text"],

Cell[StyleData["Notebook"],
  PrintingStyleEnvironment->"Printout",
  PageHeaders->{{Cell[ 
          TextData[ {
            CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
          TextData[ {
            ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
          TextData[ {
            ValueBox[ "FileName"]}], "Header"], None, Cell[ 
          TextData[ {
            CounterBox[ "Page"]}], "PageNumber"]}},
  PrintingOptions->{"PrintingMargins"->{{97, 97}, {180, 64}},
  "FirstPageHeader"->False,
  "FacingPages"->True},
  CellLabelAutoDelete->False,
  CellFrameLabelMargins->6,
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Headings", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Title"],
  CellMargins->{{12, Inherited}, {20, 40}},
  CellGroupingRules->{"TitleGrouping", 0},
  PageBreakAbove->True,
  PageBreakBelow->False,
  CounterIncrements->"Title",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subtitle", 0}, {"Subsubtitle", 0}},
  FontFamily->"Helvetica",
  FontSize->36,
  FontWeight->"Bold"],

Cell[StyleData["Title", "Printout"],
  CellMargins->{{2, 10}, {15, 30}},
  FontSize->30],

Cell[StyleData["Title", "EnhancedPrintout"],
  CellMargins->{{2, 10}, {15, 30}},
  FontFamily->"Futura",
  FontSize->30]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subtitle"],
  CellMargins->{{12, Inherited}, {10, 2}},
  CellGroupingRules->{"TitleGrouping", 10},
  PageBreakBelow->False,
  CounterIncrements->"Subtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subsubtitle", 0}},
  FontFamily->"Helvetica",
  FontSize->28],

Cell[StyleData["Subtitle", "Printout"],
  CellMargins->{{2, 10}, {10, 15}},
  FontSize->24],

Cell[StyleData["Subtitle", "EnhancedPrintout"],
  CellMargins->{{2, 10}, {10, 15}},
  FontFamily->"Futura",
  FontSize->24]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsubtitle"],
  CellMargins->{{12, Inherited}, {10, 20}},
  CellGroupingRules->{"TitleGrouping", 20},
  PageBreakBelow->False,
  CounterIncrements->"Subsubtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
  FontFamily->"Helvetica",
  FontSize->14,
  FontSlant->"Italic"],

Cell[StyleData["Subsubtitle", "Printout"],
  CellMargins->{{2, 10}, {8, 10}},
  FontSize->12],

Cell[StyleData["Subsubtitle", "EnhancedPrintout"],
  CellMargins->{{2, 10}, {8, 10}},
  FontFamily->"Futura",
  FontSize->12]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Section"],
  CellFrame->{{0, 0}, {0.5, 0}},
  CellMargins->{{10, Inherited}, {8, 30}},
  CellGroupingRules->{"SectionGrouping", 30},
  PageBreakBelow->False,
  CellFrameMargins->8,
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
  FontFamily->"Helvetica",
  FontSize->20,
  FontWeight->"Bold",
  CellTags->"Section"],

Cell[StyleData["Section", "Printout"],
  CellMargins->{{2, 0}, {7, 24}},
  FontSize->16],

Cell[StyleData["Section", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {7, 24}},
  FontFamily->"Futura",
  FontSize->16]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsection"],
  CellDingbat->"\[FilledSquare]",
  CellMargins->{{24, Inherited}, {8, 30}},
  CellGroupingRules->{"SectionGrouping", 40},
  PageBreakBelow->False,
  CounterIncrements->"Subsection",
  CounterAssignments->{{"Subsubsection", 0}},
  FontFamily->"Helvetica",
  FontSize->14,
  FontWeight->"Bold",
  CellTags->"Subsection"],

Cell[StyleData["Subsection", "Printout"],
  CellMargins->{{9, 0}, {7, 24}},
  FontSize->12],

Cell[StyleData["Subsection", "EnhancedPrintout"],
  CellMargins->{{9, 0}, {7, 24}},
  FontFamily->"Futura",
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsubsection"],
  CellMargins->{{12, Inherited}, {8, 24}},
  CellGroupingRules->{"SectionGrouping", 50},
  PageBreakBelow->False,
  CounterIncrements->"Subsubsection",
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  CellTags->"Subsubsection"],

Cell[StyleData["Subsubsection", "Printout"],
  CellMargins->{{2, 0}, {7, 18}},
  FontSize->10],

Cell[StyleData["Subsubsection", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {7, 18}},
  FontFamily->"Futura",
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Body Text", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Text"],
  CellMargins->{{12, 10}, {7, 7}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0.7, 0},
  CounterIncrements->"Text"],

Cell[StyleData["Text", "Printout"],
  CellMargins->{{2, 2}, {6, 6}},
  TextJustification->1,
  FontSize->11],

Cell[StyleData["Text", "EnhancedPrintout"],
  CellMargins->{{2, 2}, {6, 6}},
  TextJustification->1,
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["SmallText"],
  CellMargins->{{12, 10}, {6, 6}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0.7, 0},
  CounterIncrements->"SmallText",
  FontSize->9],

Cell[StyleData["SmallText", "Printout"],
  CellMargins->{{2, 2}, {5, 5}},
  TextJustification->1,
  FontSize->7],

Cell[StyleData["SmallText", "EnhancedPrintout"],
  CellMargins->{{2, 2}, {5, 5}},
  TextJustification->1,
  FontFamily->"Palatino",
  FontSize->7]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Input/Output", "Section"],

Cell["\<\
The cells in this section define styles used for input and output \
to the kernel.  Be careful when modifying, renaming, or removing these \
styles, because the front end associates special meanings with these style \
names.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["Input"],
  ShowCellBracket->True,
  CellMargins->{{45, 10}, {5, 7}},
  Evaluatable->True,
  CellGroupingRules->"InputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  DefaultFormatType->DefaultInputFormatType,
  AutoItalicWords->{},
  FormatType->InputForm,
  ShowStringCharacters->True,
  NumberMarks->True,
  CounterIncrements->"Input",
  FontWeight->"Bold"],

Cell[StyleData["Input", "Printout"],
  ShowCellBracket->False,
  CellMargins->{{39, 0}, {4, 6}},
  FontSize->9],

Cell[StyleData["Input", "EnhancedPrintout"],
  ShowCellBracket->False,
  CellMargins->{{39, 0}, {4, 6}},
  FontSize->9]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Output"],
  ShowCellBracket->True,
  CellMargins->{{46, 10}, {7, 5}},
  CellEditDuplicate->True,
  CellGroupingRules->"OutputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  DefaultFormatType->DefaultOutputFormatType,
  AutoItalicWords->{},
  FormatType->InputForm,
  CounterIncrements->"Output"],

Cell[StyleData["Output", "Printout"],
  ShowCellBracket->False,
  CellMargins->{{39, 0}, {6, 4}},
  FontSize->9],

Cell[StyleData["Output", "EnhancedPrintout"],
  ShowCellBracket->False,
  CellMargins->{{39, 0}, {6, 4}},
  FontSize->9]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Message"],
  ShowCellBracket->True,
  CellMargins->{{45, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"OutputGrouping",
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  AutoItalicWords->{},
  FormatType->InputForm,
  CounterIncrements->"Message",
  StyleMenuListing->None,
  FontSlant->"Italic",
  FontColor->RGBColor[1, 0, 0]],

Cell[StyleData["Message", "Printout"],
  ShowCellBracket->False,
  CellMargins->{{39, Inherited}, {Inherited, Inherited}},
  FontSize->9,
  FontColor->GrayLevel[0]],

Cell[StyleData["Message", "EnhancedPrintout"],
  ShowCellBracket->False,
  CellMargins->{{39, Inherited}, {Inherited, Inherited}},
  FontFamily->"Courier",
  FontSize->9,
  FontColor->GrayLevel[0]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Print"],
  ShowCellBracket->True,
  CellMargins->{{75, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"OutputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  AutoItalicWords->{},
  FormatType->InputForm,
  CounterIncrements->"Print",
  StyleMenuListing->None],

Cell[StyleData["Print", "Printout"],
  ShowCellBracket->False,
  CellMargins->{{57, Inherited}, {Inherited, Inherited}},
  FontSize->9],

Cell[StyleData["Print", "EnhancedPrintout"],
  ShowCellBracket->False,
  CellMargins->{{57, Inherited}, {Inherited, Inherited}},
  FontFamily->"Courier",
  FontSize->9]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Graphics"],
  ShowCellBracket->True,
  CellMargins->{{47, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"GraphicsGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  FormatType->InputForm,
  CounterIncrements->"Graphics",
  StyleMenuListing->None],

Cell[StyleData["Graphics", "Printout"],
  ShowCellBracket->False,
  CellMargins->{{39, Inherited}, {Inherited, Inherited}},
  ImageSize->{250, 250},
  FontSize->9],

Cell[StyleData["Graphics", "EnhancedPrintout"],
  ShowCellBracket->False,
  CellMargins->{{39, Inherited}, {Inherited, Inherited}},
  ImageSize->{250, 250},
  FontFamily->"Courier",
  FontSize->9]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["CellLabel"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->9,
  FontColor->RGBColor[0, 0, 1]],

Cell[StyleData["CellLabel", "Printout"],
  FontFamily->"Courier",
  FontSize->8,
  FontSlant->"Italic",
  FontColor->GrayLevel[0]],

Cell[StyleData["CellLabel", "EnhancedPrintout"],
  FontFamily->"Courier",
  FontSize->8,
  FontSlant->"Italic",
  FontColor->GrayLevel[0]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Inline Formatting", "Section"],

Cell["\<\
These styles are for modifying individual words or letters in a \
cell exclusive of the cell tag.\
\>", "Text"],

Cell[StyleData["RM"],
  StyleMenuListing->None,
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["BF"],
  StyleMenuListing->None,
  FontWeight->"Bold"],

Cell[StyleData["IT"],
  StyleMenuListing->None,
  FontSlant->"Italic"],

Cell[StyleData["TR"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["TI"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["TB"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Bold",
  FontSlant->"Plain"],

Cell[StyleData["TBI"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[StyleData["MR"],
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["MO"],
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["MB"],
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Bold",
  FontSlant->"Plain"],

Cell[StyleData["MBO"],
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[StyleData["SR"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["SO"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["SB"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  FontSlant->"Plain"],

Cell[StyleData["SBO"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[CellGroupData[{

Cell[StyleData["SO10"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->10,
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["SO10", "Printout"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->7,
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["SO10", "EnhancedPrintout"],
  StyleMenuListing->None,
  FontFamily->"Futura",
  FontSize->7,
  FontWeight->"Plain",
  FontSlant->"Italic"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Miscellaneous Styles", "Section"],

Cell[CellGroupData[{

Cell[StyleData["MathCaption"],
  CellDingbat->"\[FilledSmallSquare]",
  CellMargins->{{46, 42}, {0, 14}},
  PageBreakBelow->False,
  LineSpacing->{1, 1},
  ParagraphSpacing->{0, 8},
  FontColor->RGBColor[0, 0, 0.6]],

Cell[StyleData["MathCaption", "Printout"],
  CellMargins->{{42, 42}, {0, 14}},
  FontSize->10,
  FontColor->GrayLevel[0]],

Cell[StyleData["MathCaption", "EnhancedPrintout"],
  CellMargins->{{42, 42}, {0, 14}},
  FontFamily->"Palatino",
  FontSize->10,
  FontColor->GrayLevel[0]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Commentary"],
  CellFrame->{{2, 0}, {0, 0}},
  CellMargins->{{36, 10}, {7, 7}},
  PageBreakBelow->False,
  CellFrameMargins->8,
  CellFrameColor->RGBColor[0, 0.2, 1],
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  FontSlant->"Italic"],

Cell[StyleData["Commentary", "Printout"],
  CellMargins->{{36, 0}, {6, 6}},
  CellFrameColor->GrayLevel[0.8],
  FontSize->10],

Cell[StyleData["Commentary", "EnhancedPrintout"],
  CellMargins->{{36, 0}, {6, 6}},
  CellFrameColor->GrayLevel[0.8],
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ItemizedText"],
  CellMargins->{{20, 4}, {0, 4}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 4},
  ParagraphIndent->-21,
  CounterIncrements->"ItemizedText"],

Cell[StyleData["ItemizedText", "Printout"],
  ParagraphIndent->-18,
  FontSize->11],

Cell[StyleData["ItemizedText", "EnhancedPrintout"],
  ParagraphIndent->-18,
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ItemizedTextNote"],
  CellMargins->{{41, 4}, {0, 4}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 4},
  CounterIncrements->"Text"],

Cell[StyleData["ItemizedTextNote", "Printout"],
  CellMargins->{{38, 4}, {0, 4}},
  FontSize->11],

Cell[StyleData["ItemizedTextNote", "EnhancedPrintout"],
  CellMargins->{{38, 4}, {0, 4}},
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["IndentedText"],
  CellMargins->{{20, 4}, {0, 6}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterIncrements->"Text"],

Cell[StyleData["IndentedText", "Printout"],
  FontSize->10],

Cell[StyleData["IndentedText", "EnhancedPrintout"],
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Emphasis Boxes and Pictures", "Section"],

Cell[CellGroupData[{

Cell[StyleData["DefinitionBox"],
  CellFrame->0.5,
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.3, 0.7},
  ColumnAlignments->{Left}}],

Cell[StyleData["DefinitionBox", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["DefinitionBox", "EnhancedPrintout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["DefinitionBox3Col"],
  CellFrame->0.5,
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.2, 0.3, 0.5},
  ColumnAlignments->{Left}}],

Cell[StyleData["DefinitionBox3Col", "Printout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["DefinitionBox3Col", "EnhancedPrintout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["DefinitionBox4Col"],
  CellFrame->0.5,
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.15, 0.35, 0.15, 0.35},
  ColumnAlignments->{Left}}],

Cell[StyleData["DefinitionBox4Col", "Printout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["DefinitionBox4Col", "EnhancedPrintout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["DefinitionBox5Col"],
  CellFrame->0.5,
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->0.2,
  ColumnAlignments->{Left}}],

Cell[StyleData["DefinitionBox5Col", "Printout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["DefinitionBox5Col", "EnhancedPrintout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["DefinitionBox6Col"],
  CellFrame->0.5,
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.13, 0.24, 0.13, 0.13, 0.24, 0.13},
  ColumnAlignments->{Left}}],

Cell[StyleData["DefinitionBox6Col", "Printout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["DefinitionBox6Col", "EnhancedPrintout"],
  CellMargins->{{2, 4}, {0, 8}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["TopBox"],
  CellFrame->{{0.5, 0.5}, {0, 0.5}},
  CellMargins->{{11, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakBelow->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.31, 0.62},
  ColumnAlignments->{Left}}],

Cell[StyleData["TopBox", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["TopBox", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["MiddleBox"],
  CellFrame->{{0.5, 0.5}, {0, 0}},
  CellMargins->{{11, 4}, {0, -7}},
  CellHorizontalScrolling->True,
  PageBreakAbove->False,
  PageBreakBelow->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.31, 0.62},
  ColumnAlignments->{Left}}],

Cell[StyleData["MiddleBox", "Printout"],
  CellMargins->{{2, 0}, {0, 2}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["MiddleBox", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 4}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["MiddleSpacedBox"],
  CellFrame->{{0.5, 0.5}, {0, 0}},
  CellMargins->{{11, 4}, {0, -7}},
  CellHorizontalScrolling->True,
  PageBreakAbove->False,
  PageBreakBelow->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.31, 0.62},
  ColumnAlignments->{Left},
  RowMinHeight->1.2}],

Cell[StyleData["MiddleSpacedBox", "Printout"],
  CellMargins->{{2, 0}, {0, 0}},
  FontSize->10,
  Background->GrayLevel[1],
  GridBoxOptions->{RowMinHeight->1.8}],

Cell[StyleData["MiddleSpacedBox", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 0}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1],
  GridBoxOptions->{RowMinHeight->1.8}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["BottomBox"],
  CellFrame->{{0.5, 0.5}, {0.5, 0}},
  CellMargins->{{11, 4}, {0, -7}},
  CellHorizontalScrolling->True,
  PageBreakAbove->False,
  PageBreakBelow->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.31, 0.62},
  ColumnAlignments->{Left},
  RowMinHeight->1.2}],

Cell[StyleData["BottomBox", "Printout"],
  CellMargins->{{2, 0}, {0, -5}},
  FontSize->10,
  Background->GrayLevel[1],
  GridBoxOptions->{RowMinHeight->2.2}],

Cell[StyleData["BottomBox", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, -5}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1],
  GridBoxOptions->{RowMinHeight->2.2}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["TopSpanBox"],
  CellFrame->{{0.5, 0.5}, {0, 0.5}},
  CellMargins->{{11, 4}, {-2, 8}},
  CellHorizontalScrolling->True,
  PageBreakBelow->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.9, 0.03},
  ColumnAlignments->{Left}}],

Cell[StyleData["TopSpanBox", "Printout"],
  CellMargins->{{2, 0}, {-2, 8}},
  FontSize->10,
  Background->GrayLevel[1]],

Cell[StyleData["TopSpanBox", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {-4, 8}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["MiddleSpanBox"],
  CellFrame->{{0.5, 0.5}, {0, 0}},
  CellMargins->{{11, 4}, {-3, -8}},
  CellHorizontalScrolling->True,
  PageBreakAbove->False,
  PageBreakBelow->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  FontWeight->"Plain",
  Background->RGBColor[1, 0.6, 0.6],
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->1,
  ColumnWidths->{0.9, 0.03},
  ColumnAlignments->{Left},
  RowMinHeight->1.2}],

Cell[StyleData["MiddleSpanBox", "Printout"],
  CellMargins->{{2, 0}, {-5, 0}},
  FontSize->10,
  Background->GrayLevel[1],
  GridBoxOptions->{RowMinHeight->1.8}],

Cell[StyleData["MiddleSpanBox", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {-7, 0}},
  FontFamily->"Palatino",
  FontSize->10,
  Background->GrayLevel[1],
  GridBoxOptions->{RowMinHeight->1.8}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Picture"],
  CellMargins->{{12, Inherited}, {0, 8}},
  CellHorizontalScrolling->True],

Cell[StyleData["Picture", "Printout"],
  CellMargins->{{2, Inherited}, {0, 8}}],

Cell[StyleData["Picture", "EnhancedPrintout"],
  CellMargins->{{2, Inherited}, {0, 8}}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Caption"],
  CellMargins->{{12, 4}, {0, 2}},
  PageBreakAbove->False,
  FontFamily->"Helvetica",
  FontSize->9],

Cell[StyleData["Caption", "Printout"],
  CellMargins->{{2, 4}, {2, 4}},
  FontSize->7],

Cell[StyleData["Caption", "EnhancedPrintout"],
  CellMargins->{{2, 4}, {2, 4}},
  FontFamily->"Futura",
  FontSize->7]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Tables", "Section"],

Cell[CellGroupData[{

Cell[StyleData["SingleRowTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnAlignments->{Left}}],

Cell[StyleData["SingleRowTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["SingleRowTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["2ColumnTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->{0.34, 0.64},
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["2ColumnTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["2ColumnTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["2ColumnEvenTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->0.49,
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["2ColumnEvenTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["2ColumnEvenTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["3ColumnTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->{0.28, 0.28, 0.43},
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["3ColumnTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["3ColumnTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["4ColumnTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->0.25,
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["4ColumnTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["4ColumnTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["5ColumnTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->0.2,
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["5ColumnTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["5ColumnTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["6ColumnTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->0.16,
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["6ColumnTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["6ColumnTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["7ColumnTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->0.14,
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["7ColumnTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["7ColumnTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["8ColumnTable"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  AutoIndent->False,
  AutoSpacing->False,
  LineIndent->0,
  StyleMenuListing->None,
  GridBoxOptions->{RowSpacings->1.5,
  ColumnSpacings->2,
  ColumnWidths->0.12,
  ColumnAlignments->{Left, Center}}],

Cell[StyleData["8ColumnTable", "Printout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10],

Cell[StyleData["8ColumnTable", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 8}},
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Automatic Numbering", "Section"],

Cell["\<\
The following styles are useful for numbered equations, figures, \
etc.  They automatically give the cell a FrameLabel containing a reference to \
a particular counter, and also increment that counter.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["NumberedEquation"],
  CellMargins->{{45, 10}, {7, 7}},
  CellFrameLabels->{{None, Cell[ 
          TextData[ {"(", 
            CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
  DefaultFormatType->DefaultInputFormatType,
  CounterIncrements->"NumberedEquation",
  FormatTypeAutoConvert->False],

Cell[StyleData["NumberedEquation", "Printout"],
  CellMargins->{{39, 55}, {0, 10}},
  FontSize->10],

Cell[StyleData["NumberedEquation", "EnhancedPrintout"],
  CellMargins->{{39, 55}, {0, 10}},
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Formulas and Programming", "Section"],

Cell[CellGroupData[{

Cell[StyleData["DisplayFormula"],
  CellMargins->{{45, Inherited}, {7, 7}},
  CellHorizontalScrolling->True,
  ScriptLevel->0,
  SingleLetterItalics->True,
  UnderoverscriptBoxOptions->{LimitsPositioning->True}],

Cell[StyleData["DisplayFormula", "Printout"],
  FontSize->10],

Cell[StyleData["DisplayFormula", "EnhancedPrintout"],
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["DisplayMath"],
  CellMargins->{{45, 10}, {7, 7}},
  Evaluatable->False,
  CellGroupingRules->"InputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  DefaultFormatType->DefaultInputFormatType,
  AutoItalicWords->{},
  FormatType->InputForm,
  ShowStringCharacters->True,
  NumberMarks->True,
  CounterIncrements->"Input"],

Cell[StyleData["DisplayMath", "Printout"],
  CellMargins->{{39, 0}, {6, 6}},
  FontSize->9],

Cell[StyleData["DisplayMath", "EnhancedPrintout"],
  CellMargins->{{39, 0}, {4, 6}},
  FontFamily->"WRICourier",
  FontSize->9]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["EquationNumber"],
  LineSpacing->{1.83, 0}],

Cell[StyleData["EquationNumber", "Printout"],
  LineSpacing->{1.7, 0},
  FontSize->10],

Cell[StyleData["EquationNumber", "EnhancedPrintout"],
  LineSpacing->{1.7, 0},
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Program"],
  CellFrame->{{0, 0}, {0.5, 0.5}},
  CellMargins->{{12, Inherited}, {7, 7}},
  CellFrameMargins->{{0, 0}, {7, 5}},
  FontFamily->"Courier"],

Cell[StyleData["Program", "Printout"],
  CellMargins->{{18, 30}, {Inherited, 4}},
  FontSize->9.5],

Cell[StyleData["Program", "EnhancedPrintout"],
  CellMargins->{{18, 30}, {Inherited, 4}},
  FontFamily->"WRICourier",
  FontSize->9.5]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Contents and Index", "Section"],

Cell[CellGroupData[{

Cell[StyleData["ContentsTitle"],
  ShowGroupOpenCloseIcon->False,
  CellMargins->{{10, 4}, {0, 18}},
  CellGroupingRules->{"SectionGrouping", 60},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->26,
  FontWeight->"Bold"],

Cell[StyleData["ContentsTitle", "Printout"],
  CellMargins->{{2, 0}, {0, 18}},
  PageBreakBelow->False,
  FontSize->18],

Cell[StyleData["ContentsTitle", "EnhancedPrintout"],
  CellMargins->{{2, 0}, {0, 18}},
  PageBreakBelow->False,
  FontFamily->"Futura",
  FontSize->18]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ContentsSection"],
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{35, 4}, {3, 18}},
  CellGroupingRules->{"SectionGrouping", 70},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->12,
  FontWeight->"Bold"],

Cell[StyleData["ContentsSection", "Printout"],
  CellMargins->{{20, 0}, {3, 18}},
  PageBreakBelow->False,
  FontSize->11],

Cell[StyleData["ContentsSection", "EnhancedPrintout"],
  CellMargins->{{20, 0}, {3, 18}},
  PageBreakBelow->False,
  FontFamily->"Futura",
  FontSize->11]
}, Open  ]],

Cell[CellGroupData[{

Cell[StyleData["Contents"],
  ShowGroupOpenCloseIcon->False,
  CellMargins->{{50, 4}, {0, 5}},
  CellGroupingRules->{"SectionGrouping", 90},
  StyleMenuListing->None],

Cell[StyleData["Contents", "Printout"],
  CellMargins->{{40, 4}, {0, 5}},
  FontSize->11],

Cell[StyleData["Contents", "EnhancedPrintout"],
  CellMargins->{{40, 5}, {0, 5}},
  FontFamily->"Palatino",
  FontSize->11]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Index"],
  CellMargins->{{21, 4}, {0, 0}},
  ParagraphIndent->-48,
  StyleMenuListing->None],

Cell[StyleData["Index", "Printout"],
  CellMargins->{{13, 0}, {0, 0}},
  FontSize->10],

Cell[StyleData["Index", "EnhancedPrintout"],
  CellMargins->{{13, 0}, {0, 0}},
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["IndexSubentry"],
  CellMargins->{{36, 4}, {0, 0}},
  ParagraphIndent->-48,
  StyleMenuListing->None],

Cell[StyleData["IndexSubentry", "Printout"],
  CellMargins->{{24, 0}, {0, 0}},
  FontSize->10],

Cell[StyleData["IndexSubentry", "EnhancedPrintout"],
  CellMargins->{{24, 0}, {0, 0}},
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Function List", "Section"],

Cell[CellGroupData[{

Cell[StyleData["FunctionName"],
  CellDingbat->"\[FilledSquare]",
  CellMargins->{{24, 10}, {0, 12}},
  CellGroupingRules->{"SectionGrouping", 40},
  PageBreakBelow->False,
  FontFamily->"Courier",
  FontSize->14,
  FontWeight->"Bold",
  CellTags->"FunctionName"],

Cell[StyleData["FunctionName", "Printout"],
  CellMargins->{{9, 0}, {0, 10}},
  FontSize->12],

Cell[StyleData["FunctionName", "EnhancedPrintout"],
  CellMargins->{{9, 0}, {0, 10}},
  FontFamily->"Courier",
  FontSize->12]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["FunctionDescription"],
  CellMargins->{{36, 10}, {4, 0}},
  LineSpacing->{1, 3},
  ParagraphSpacing->{0.7, 0},
  CounterIncrements->"Text"],

Cell[StyleData["FunctionDescription", "Printout"],
  CellMargins->{{36, 0}, {4, 0}},
  FontSize->10],

Cell[StyleData["FunctionDescription", "EnhancedPrintout"],
  CellMargins->{{36, 0}, {4, 0}},
  TextJustification->0,
  FontFamily->"Palatino",
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["FunctionExample"],
  CellMargins->{{36, 10}, {4, 0}},
  StyleMenuListing->None,
  FontSize->10],

Cell[StyleData["FunctionExample", "Printout"],
  CellMargins->{{36, 0}, {4, 0}},
  FontSize->9],

Cell[StyleData["FunctionExample", "EnhancedPrintout"],
  CellMargins->{{36, 0}, {4, 0}},
  FontFamily->"Palatino",
  FontSize->9]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Browser Emulation", "Section"],

Cell[CellGroupData[{

Cell[StyleData["TextNormal"],
  CellMargins->{{12, 10}, {7, 7}},
  LineSpacing->{1, 3},
  CounterIncrements->"Text",
  FontFamily->"Times",
  FontSize->12],

Cell[StyleData["TextNormal", "Printout"],
  CellMargins->{{2, 2}, {6, 6}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["BrowserEmulation"],
  CellMargins->{{12, 1}, {7, 24}},
  CellGroupingRules->{"SectionGrouping", 70},
  LineSpacing->{1, 3},
  FontFamily->"Helvetica",
  GridBoxOptions->{ColumnWidths->0.25,
  ColumnAlignments->{Left},
  GridFrame->True,
  GridFrameMargins->0.5,
  ColumnLines->True}],

Cell[StyleData["Text", "Printout"],
  CellMargins->{{2, 2}, {6, 6}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["CategoryDefs"],
  CellMargins->{{300, 10}, {7, 7}},
  LineSpacing->{1, 3},
  FontFamily->"Helvetica",
  Background->RGBColor[1, 0.8, 0.6],
  GridBoxOptions->{ColumnAlignments->{Left},
  GridFrame->False,
  RowLines->True,
  ColumnLines->True}],

Cell[StyleData["Text", "Printout"],
  CellMargins->{{2, 2}, {6, 6}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Category1"],
  ShowCellBracket->False,
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{12, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 50},
  FontSize->16,
  FontWeight->"Bold"],

Cell[StyleData["Category1", "Printout"],
  CellMargins->{{2, 2}, {6, 6}}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Category2"],
  ShowCellBracket->False,
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{85, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 60},
  FontSize->14,
  FontWeight->"Bold",
  FontColor->RGBColor[0, 0, 0.500008]],

Cell[StyleData["Category2", "Printout"],
  CellMargins->{{2, 2}, {6, 6}}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Category3"],
  ShowCellBracket->False,
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{157, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 70},
  FontSize->12,
  FontWeight->"Bold",
  FontColor->RGBColor[0.2, 0, 0.4]],

Cell[StyleData["Category3", "Printout"],
  CellMargins->{{2, 2}, {6, 6}}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Category4"],
  ShowCellBracket->False,
  ShowGroupOpenCloseIcon->True,
  CellMargins->{{230, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 80},
  FontWeight->"Bold",
  FontColor->RGBColor[0, 0.300008, 0.4]],

Cell[StyleData["Category4", "Printout"],
  CellMargins->{{2, 2}, {6, 6}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Headers and Footers", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Header"],
  CellMargins->{{0, 0}, {4, 1}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->8,
  FontWeight->"Bold"],

Cell[StyleData["Header", "Printout"]],

Cell[StyleData["Header", "EnhancedPrintout"],
  FontFamily->"Futura"]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Footer"],
  CellMargins->{{0, 0}, {0, 4}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->7,
  FontSlant->"Plain"],

Cell[StyleData["Footer", "Printout"]],

Cell[StyleData["Footer", "EnhancedPrintout"],
  FontFamily->"Futura"]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["PageNumber"],
  CellMargins->{{0, 0}, {4, 1}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->8,
  FontWeight->"Bold"],

Cell[StyleData["PageNumber", "Printout"]],

Cell[StyleData["PageNumber", "EnhancedPrintout"],
  FontFamily->"Futura"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Hyperlink Styles", "Section"],

Cell["\<\
The cells below define styles useful for making hypertext \
ButtonBoxes.  The \"Hyperlink\" style is for links within the same Notebook, \
or between Notebooks.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["Hyperlink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`NotebookLocate[ #2]}]&),
  Active->True,
  ButtonNote->ButtonData}],

Cell[StyleData["Hyperlink", "Printout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}],

Cell[StyleData["Hyperlink", "EnhancedPrintout"],
  FontFamily->"Palatino",
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell["\<\
The following styles are for linking automatically to the on-line \
help system.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["MainBookLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["MainBookLink", "Printout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}],

Cell[StyleData["MainBookLink", "EnhancedPrintout"],
  FontFamily->"Palatino",
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["AddOnsLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontFamily->"Courier",
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["AddOnsLink", "Printout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}],

Cell[StyleData["AddOnsLink", "EnhancedPrintout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["AddOnsLinkText"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontFamily->"Times",
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["AddOnsLinkText", "Printout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}],

Cell[StyleData["AddOnsLinkText", "EnhancedPrintout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["RefGuideLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontFamily->"Courier",
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "RefGuide", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["RefGuideLink", "Printout"],
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}],

Cell[StyleData["RefGuideLink", "EnhancedPrintout"],
  FontFamily->"WRICourier",
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["GettingStartedLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["GettingStartedLink", "Printout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}],

Cell[StyleData["GettingStartedLink", "EnhancedPrintout"],
  FontFamily->"Palatino",
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["OtherInformationLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0, 0, 1],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["OtherInformationLink", "Printout"],
  FontSize->11,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}],

Cell[StyleData["OtherInformationLink", "EnhancedPrintout"],
  FontFamily->"Palatino",
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Outline Styles", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Outline1"],
  CellMargins->{{12, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 50},
  ParagraphIndent->-38,
  CounterIncrements->"Outline1",
  CounterBoxOptions->{CounterFunction:>CapitalRomanNumeral},
  FontSize->18,
  FontWeight->"Bold"],

Cell[StyleData["Outline1", "Printout"],
  CounterBoxOptions->{CounterFunction:>CapitalRomanNumeral}]
}, Open  ]],

Cell[CellGroupData[{

Cell[StyleData["Outline2"],
  CellMargins->{{59, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 60},
  ParagraphIndent->-27,
  CounterIncrements->"Outline2",
  CounterBoxOptions->{CounterFunction:>(Part[ 
      CharacterRange[ "A", "Z"], #]&)},
  FontSize->15,
  FontWeight->"Bold"],

Cell[StyleData["Outline2", "Printout"],
  CounterBoxOptions->{CounterFunction:>(Part[ 
      CharacterRange[ "A", "Z"], #]&)}]
}, Open  ]],

Cell[CellGroupData[{

Cell[StyleData["Outline3"],
  CellMargins->{{108, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 70},
  ParagraphIndent->-21,
  CounterIncrements->"Outline3",
  CounterBoxOptions->{CounterFunction:>Identity},
  FontSize->12],

Cell[StyleData["Outline3", "Printout"],
  CounterBoxOptions->{CounterFunction:>Identity}]
}, Open  ]],

Cell[CellGroupData[{

Cell[StyleData["Outline4"],
  CellMargins->{{158, 10}, {7, 7}},
  CellGroupingRules->{"SectionGrouping", 80},
  ParagraphIndent->-18,
  CounterIncrements->"Outline4",
  CounterBoxOptions->{CounterFunction:>(Part[ 
      CharacterRange[ "a", "z"], #]&)},
  FontSize->10],

Cell[StyleData["Outline4", "Printout"]]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1024}, {0, 748}},
WindowSize->{489, 608},
WindowMargins->{{20, Automatic}, {Automatic, -1}},
MacintoshSystemPageSetup->"\<\
00<0001804P000000]P2:?oQon82n@960dL5:0?l0080001804P000000]P2:001
0000I00000400`<300000BL?00400@0000000000000006P801T1T00000000000
00000000000000000000000000000000\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Section"->{
    Cell[5550, 197, 381, 11, 56, 26, 0, "StyleData", "Section", "All",
      CounterIncrements->"Section",
      CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
      CellTags->"Section"]},
  "Subsection"->{
    Cell[6182, 222, 349, 10, 44, 29, 0, "StyleData", "Subsection", "All",
      CounterIncrements->"Subsection",
      CounterAssignments->{{"Subsubsection", 0}},
      CellTags->"Subsection"]},
  "Subsubsection"->{
    Cell[6788, 246, 262, 7, 42, 32, 0, "StyleData", "Subsubsection", "All",
      CounterIncrements->"Subsubsection",
      CellTags->"Subsubsection"]},
  "FunctionName"->{
    Cell[37281, 1538, 263, 8, 42, 31, 0, "StyleData", "FunctionName", "All",
      CellTags->"FunctionName"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Section", 49534, 2028},
  {"Subsection", 49757, 2033},
  {"Subsubsection", 49973, 2038},
  {"FunctionName", 50146, 2042}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 37, 0, 62, "Subtitle"],
Cell[1771, 53, 193, 4, 46, "Text"],

Cell[CellGroupData[{
Cell[1989, 61, 42, 0, 50, "Section"],
Cell[2034, 63, 166, 4, 36, 31, 0, "StyleData", "All", "Working"],
Cell[2203, 69, 106, 3, 36, 35, 0, "StyleData", "All", "AllBrackets"],
Cell[2312, 74, 140, 4, 36, 36, 0, "StyleData", "All", "Presentation"],
Cell[2455, 80, 212, 5, 36, 32, 0, "StyleData", "All", "Printout"],
Cell[2670, 87, 220, 5, 36, 40, 0, "StyleData", "All", "EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[2927, 97, 35, 0, 30, "Section"],
Cell[2965, 99, 101, 3, 30, "Text"],
Cell[3069, 104, 610, 16, 36, 27, 0, "StyleData", "Notebook", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[3716, 125, 38, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[3779, 129, 371, 10, 112, 24, 0, "StyleData", "Title", "All",
  CounterIncrements->"Title",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subtitle", 0}, {"Subsubtitle", 0}}],
Cell[4153, 141, 88, 2, 91, 36, 0, "StyleData", "Title", "Printout"],
Cell[4244, 145, 120, 3, 94, 44, 0, "StyleData", "Title", "EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[4401, 153, 314, 8, 58, 27, 0, "StyleData", "Subtitle", "All",
  CounterIncrements->"Subtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subsubtitle", 0}}],
Cell[4718, 163, 91, 2, 66, 39, 0, "StyleData", "Subtitle", "Printout"],
Cell[4812, 167, 123, 3, 67, 47, 0, "StyleData", "Subtitle", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[4972, 175, 317, 8, 46, 30, 0, "StyleData", "Subsubtitle", "All",
  CounterIncrements->"Subsubtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}}],
Cell[5292, 185, 93, 2, 48, 42, 0, "StyleData", "Subsubtitle", "Printout"],
Cell[5388, 189, 125, 3, 47, 50, 0, "StyleData", "Subsubtitle", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[5550, 197, 381, 11, 56, 26, 0, "StyleData", "Section", "All",
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
  CellTags->"Section"],
Cell[5934, 210, 88, 2, 71, 38, 0, "StyleData", "Section", "Printout"],
Cell[6025, 214, 120, 3, 72, 46, 0, "StyleData", "Section", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[6182, 222, 349, 10, 44, 29, 0, "StyleData", "Subsection", "All",
  CounterIncrements->"Subsection",
  CounterAssignments->{{"Subsubsection", 0}},
  CellTags->"Subsection"],
Cell[6534, 234, 91, 2, 61, 41, 0, "StyleData", "Subsection", "Printout"],
Cell[6628, 238, 123, 3, 62, 49, 0, "StyleData", "Subsection", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[6788, 246, 262, 7, 42, 32, 0, "StyleData", "Subsubsection", "All",
  CounterIncrements->"Subsubsection",
  CellTags->"Subsubsection"],
Cell[7053, 255, 94, 2, 52, 44, 0, "StyleData", "Subsubsection", "Printout"],
Cell[7150, 259, 126, 3, 52, 52, 0, "StyleData", "Subsubsection", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[7325, 268, 39, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[7389, 272, 140, 4, 42, 23, 0, "StyleData", "Text", "All",
  CounterIncrements->"Text"],
Cell[7532, 278, 108, 3, 39, 35, 0, "StyleData", "Text", "Printout"],
Cell[7643, 283, 142, 4, 40, 43, 0, "StyleData", "Text", "EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[7822, 292, 165, 5, 36, 28, 0, "StyleData", "SmallText", "All",
  CounterIncrements->"SmallText"],
Cell[7990, 299, 112, 3, 33, 40, 0, "StyleData", "SmallText", "Printout"],
Cell[8105, 304, 146, 4, 34, 48, 0, "StyleData", "SmallText", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[8300, 314, 42, 0, 30, "Section"],
Cell[8345, 316, 248, 5, 62, "Text"],

Cell[CellGroupData[{
Cell[8618, 325, 398, 13, 42, 24, 0, "StyleData", "Input", "All",
  CounterIncrements->"Input"],
Cell[9019, 340, 111, 3, 36, 36, 0, "StyleData", "Input", "Printout"],
Cell[9133, 345, 119, 3, 36, 44, 0, "StyleData", "Input", "EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[9289, 353, 416, 13, 39, 25, 0, "StyleData", "Output", "All",
  CounterIncrements->"Output"],
Cell[9708, 368, 112, 3, 36, 37, 0, "StyleData", "Output", "Printout"],
Cell[9823, 373, 120, 3, 36, 45, 0, "StyleData", "Output", "EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[9980, 381, 486, 15, 38, 26, 0, "StyleData", "Message", "All",
  CounterIncrements->"Message",
  StyleMenuListing->None],
Cell[10469, 398, 164, 4, 34, 38, 0, "StyleData", "Message", "Printout"],
Cell[10636, 404, 197, 5, 34, 46, 0, "StyleData", "Message", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[10870, 414, 460, 14, 36, 24, 0, "StyleData", "Print", "All",
  CounterIncrements->"Print",
  StyleMenuListing->None],
Cell[11333, 430, 135, 3, 34, 36, 0, "StyleData", "Print", "Printout"],
Cell[11471, 435, 168, 4, 34, 44, 0, "StyleData", "Print", "EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[11676, 444, 414, 12, 36, 27, 0, "StyleData", "Graphics", "All",
  CounterIncrements->"Graphics",
  StyleMenuListing->None],
Cell[12093, 458, 163, 4, 34, 39, 0, "StyleData", "Graphics", "Printout"],
Cell[12259, 464, 196, 5, 34, 47, 0, "StyleData", "Graphics", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[12492, 474, 128, 4, 34, 28, 0, "StyleData", "CellLabel", "All",
  StyleMenuListing->None],
Cell[12623, 480, 130, 4, 32, 40, 0, "StyleData", "CellLabel", "Printout"],
Cell[12756, 486, 138, 4, 32, 48, 0, "StyleData", "CellLabel", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[12943, 496, 36, 0, 30, "Section"],
Cell[12982, 498, 121, 3, 30, "Text"],
Cell[13106, 503, 92, 3, 36, 21, 0, "StyleData", "RM", "All",
  StyleMenuListing->None],
Cell[13201, 508, 69, 2, 38, 21, 0, "StyleData", "BF", "All",
  StyleMenuListing->None],
Cell[13273, 512, 70, 2, 38, 21, 0, "StyleData", "IT", "All",
  StyleMenuListing->None],
Cell[13346, 516, 115, 4, 36, 21, 0, "StyleData", "TR", "All",
  StyleMenuListing->None],
Cell[13464, 522, 116, 4, 38, 21, 0, "StyleData", "TI", "All",
  StyleMenuListing->None],
Cell[13583, 528, 114, 4, 38, 21, 0, "StyleData", "TB", "All",
  StyleMenuListing->None],
Cell[13700, 534, 116, 4, 38, 22, 0, "StyleData", "TBI", "All",
  StyleMenuListing->None],
Cell[13819, 540, 117, 4, 36, 21, 0, "StyleData", "MR", "All",
  StyleMenuListing->None],
Cell[13939, 546, 118, 4, 37, 21, 0, "StyleData", "MO", "All",
  StyleMenuListing->None],
Cell[14060, 552, 116, 4, 37, 21, 0, "StyleData", "MB", "All",
  StyleMenuListing->None],
Cell[14179, 558, 118, 4, 37, 22, 0, "StyleData", "MBO", "All",
  StyleMenuListing->None],
Cell[14300, 564, 119, 4, 37, 21, 0, "StyleData", "SR", "All",
  StyleMenuListing->None],
Cell[14422, 570, 120, 4, 38, 21, 0, "StyleData", "SO", "All",
  StyleMenuListing->None],
Cell[14545, 576, 118, 4, 38, 21, 0, "StyleData", "SB", "All",
  StyleMenuListing->None],
Cell[14666, 582, 120, 4, 38, 22, 0, "StyleData", "SBO", "All",
  StyleMenuListing->None],

Cell[CellGroupData[{
Cell[14811, 590, 138, 5, 35, 23, 0, "StyleData", "SO10", "All",
  StyleMenuListing->None],
Cell[14952, 597, 149, 5, 31, 35, 0, "StyleData", "SO10", "Printout",
  StyleMenuListing->None],
Cell[15104, 604, 154, 5, 32, 43, 0, "StyleData", "SO10", "EnhancedPrintout",
  StyleMenuListing->None]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[15307, 615, 39, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[15371, 619, 215, 6, 42, 30, 0, "StyleData", "MathCaption", "All"],
Cell[15589, 627, 121, 3, 41, 42, 0, "StyleData", "MathCaption", "Printout"],
Cell[15713, 632, 155, 4, 42, 50, 0, "StyleData", "MathCaption", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[15905, 641, 256, 8, 41, 29, 0, "StyleData", "Commentary", "All"],
Cell[16164, 651, 125, 3, 39, 41, 0, "StyleData", "Commentary", "Printout"],
Cell[16292, 656, 159, 4, 40, 49, 0, "StyleData", "Commentary", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[16488, 665, 177, 5, 32, 31, 0, "StyleData", "ItemizedText", "All",
  CounterIncrements->"ItemizedText"],
Cell[16668, 672, 83, 2, 31, 43, 0, "StyleData", "ItemizedText", "Printout"],
Cell[16754, 676, 117, 3, 32, 51, 0, "StyleData", "ItemizedText", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[16908, 684, 149, 4, 32, 35, 0, "StyleData", "ItemizedTextNote", "All",
  CounterIncrements->"Text"],
Cell[17060, 690, 97, 2, 31, 47, 0, "StyleData", "ItemizedTextNote", 
"Printout"],
Cell[17160, 694, 131, 3, 32, 55, 0, "StyleData", "ItemizedTextNote", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[17328, 702, 145, 4, 32, 31, 0, "StyleData", "IndentedText", "All",
  CounterIncrements->"Text"],
Cell[17476, 708, 59, 1, 33, 43, 0, "StyleData", "IndentedText", "Printout"],
Cell[17538, 711, 93, 2, 34, 51, 0, "StyleData", "IndentedText", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[17680, 719, 46, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[17751, 723, 404, 14, 53, 32, 0, "StyleData", "DefinitionBox", "All",
  StyleMenuListing->None],
Cell[18158, 739, 121, 3, 52, 44, 0, "StyleData", "DefinitionBox", 
"Printout"],
Cell[18282, 744, 155, 4, 53, 52, 0, "StyleData", "DefinitionBox", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[18474, 753, 413, 14, 49, 36, 0, "StyleData", "DefinitionBox3Col", "All",
  StyleMenuListing->None],
Cell[18890, 769, 125, 3, 52, 48, 0, "StyleData", "DefinitionBox3Col", 
"Printout"],
Cell[19018, 774, 159, 4, 53, 56, 0, "StyleData", "DefinitionBox3Col", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[19214, 783, 422, 14, 49, 36, 0, "StyleData", "DefinitionBox4Col", "All",
  StyleMenuListing->None],
Cell[19639, 799, 125, 3, 52, 48, 0, "StyleData", "DefinitionBox4Col", 
"Printout"],
Cell[19767, 804, 159, 4, 53, 56, 0, "StyleData", "DefinitionBox4Col", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[19963, 813, 401, 14, 49, 36, 0, "StyleData", "DefinitionBox5Col", "All",
  StyleMenuListing->None],
Cell[20367, 829, 125, 3, 52, 48, 0, "StyleData", "DefinitionBox5Col", 
"Printout"],
Cell[20495, 834, 159, 4, 53, 56, 0, "StyleData", "DefinitionBox5Col", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[20691, 843, 434, 14, 49, 36, 0, "StyleData", "DefinitionBox6Col", "All",
  StyleMenuListing->None],
Cell[21128, 859, 125, 3, 52, 48, 0, "StyleData", "DefinitionBox6Col", 
"Printout"],
Cell[21256, 864, 159, 4, 53, 56, 0, "StyleData", "DefinitionBox6Col", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[21452, 873, 417, 14, 48, 25, 0, "StyleData", "TopBox", "All",
  StyleMenuListing->None],
Cell[21872, 889, 114, 3, 43, 37, 0, "StyleData", "TopBox", "Printout"],
Cell[21989, 894, 148, 4, 44, 45, 0, "StyleData", "TopBox", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[22174, 903, 444, 15, 48, 28, 0, "StyleData", "MiddleBox", "All",
  StyleMenuListing->None],
Cell[22621, 920, 117, 3, 29, 40, 0, "StyleData", "MiddleBox", "Printout"],
Cell[22741, 925, 151, 4, 32, 48, 0, "StyleData", "MiddleBox", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[22929, 934, 471, 16, 48, 34, 0, "StyleData", "MiddleSpacedBox", "All",
  StyleMenuListing->None],
Cell[23403, 952, 162, 4, 27, 46, 0, "StyleData", "MiddleSpacedBox", 
"Printout"],
Cell[23568, 958, 196, 5, 28, 54, 0, "StyleData", "MiddleSpacedBox", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[23801, 968, 467, 16, 48, 28, 0, "StyleData", "BottomBox", "All",
  StyleMenuListing->None],
Cell[24271, 986, 157, 4, 30, 40, 0, "StyleData", "BottomBox", "Printout"],
Cell[24431, 992, 191, 5, 31, 48, 0, "StyleData", "BottomBox", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[24659, 1002, 421, 14, 46, 29, 0, "StyleData", "TopSpanBox", "All",
  StyleMenuListing->None],
Cell[25083, 1018, 119, 3, 41, 41, 0, "StyleData", "TopSpanBox", "Printout"],
Cell[25205, 1023, 153, 4, 40, 49, 0, "StyleData", "TopSpanBox", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[25395, 1032, 469, 16, 45, 32, 0, "StyleData", "MiddleSpanBox", "All",
  StyleMenuListing->None],
Cell[25867, 1050, 161, 4, 22, 44, 0, "StyleData", "MiddleSpanBox", 
"Printout"],
Cell[26031, 1056, 195, 5, 21, 52, 0, "StyleData", "MiddleSpanBox", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[26263, 1066, 101, 2, 32, 26, 0, "StyleData", "Picture", "All"],
Cell[26367, 1070, 79, 1, 36, 38, 0, "StyleData", "Picture", "Printout"],
Cell[26449, 1073, 87, 1, 36, 46, 0, "StyleData", "Picture", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[26573, 1079, 127, 4, 30, 26, 0, "StyleData", "Caption", "All"],
Cell[26703, 1085, 86, 2, 29, 38, 0, "StyleData", "Caption", "Printout"],
Cell[26792, 1089, 118, 3, 30, 46, 0, "StyleData", "Caption", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[26959, 1098, 25, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[27009, 1102, 299, 10, 36, 33, 0, "StyleData", "SingleRowTable", "All",
  StyleMenuListing->None],
Cell[27311, 1114, 94, 2, 35, 45, 0, "StyleData", "SingleRowTable", 
"Printout"],
Cell[27408, 1118, 102, 2, 35, 53, 0, "StyleData", "SingleRowTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[27547, 1125, 335, 11, 32, 31, 0, "StyleData", "2ColumnTable", "All",
  StyleMenuListing->None],
Cell[27885, 1138, 92, 2, 35, 43, 0, "StyleData", "2ColumnTable", "Printout"],
Cell[27980, 1142, 100, 2, 35, 51, 0, "StyleData", "2ColumnTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[28117, 1149, 331, 11, 32, 35, 0, "StyleData", "2ColumnEvenTable", "All",
  StyleMenuListing->None],
Cell[28451, 1162, 96, 2, 35, 47, 0, "StyleData", "2ColumnEvenTable", 
"Printout"],
Cell[28550, 1166, 104, 2, 35, 55, 0, "StyleData", "2ColumnEvenTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[28691, 1173, 341, 11, 32, 31, 0, "StyleData", "3ColumnTable", "All",
  StyleMenuListing->None],
Cell[29035, 1186, 92, 2, 35, 43, 0, "StyleData", "3ColumnTable", "Printout"],
Cell[29130, 1190, 100, 2, 35, 51, 0, "StyleData", "3ColumnTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[29267, 1197, 327, 11, 32, 31, 0, "StyleData", "4ColumnTable", "All",
  StyleMenuListing->None],
Cell[29597, 1210, 92, 2, 35, 43, 0, "StyleData", "4ColumnTable", "Printout"],
Cell[29692, 1214, 100, 2, 35, 51, 0, "StyleData", "4ColumnTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[29829, 1221, 326, 11, 32, 31, 0, "StyleData", "5ColumnTable", "All",
  StyleMenuListing->None],
Cell[30158, 1234, 92, 2, 35, 43, 0, "StyleData", "5ColumnTable", "Printout"],
Cell[30253, 1238, 100, 2, 35, 51, 0, "StyleData", "5ColumnTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[30390, 1245, 327, 11, 32, 31, 0, "StyleData", "6ColumnTable", "All",
  StyleMenuListing->None],
Cell[30720, 1258, 92, 2, 35, 43, 0, "StyleData", "6ColumnTable", "Printout"],
Cell[30815, 1262, 100, 2, 35, 51, 0, "StyleData", "6ColumnTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[30952, 1269, 327, 11, 32, 31, 0, "StyleData", "7ColumnTable", "All",
  StyleMenuListing->None],
Cell[31282, 1282, 92, 2, 35, 43, 0, "StyleData", "7ColumnTable", "Printout"],
Cell[31377, 1286, 100, 2, 35, 51, 0, "StyleData", "7ColumnTable", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[31514, 1293, 327, 11, 32, 31, 0, "StyleData", "8ColumnTable", "All",
  StyleMenuListing->None],
Cell[31844, 1306, 92, 2, 35, 43, 0, "StyleData", "8ColumnTable", "Printout"],
Cell[31939, 1310, 100, 2, 35, 51, 0, "StyleData", "8ColumnTable", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[32088, 1318, 49, 0, 30, "Section"],
Cell[32140, 1320, 225, 4, 62, "Text"],

Cell[CellGroupData[{
Cell[32390, 1328, 318, 7, 42, 35, 0, "StyleData", "NumberedEquation", "All",
  CounterIncrements->"NumberedEquation"],
Cell[32711, 1337, 99, 2, 37, 47, 0, "StyleData", "NumberedEquation", 
"Printout"],
Cell[32813, 1341, 133, 3, 38, 55, 0, "StyleData", "NumberedEquation", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[32995, 1350, 43, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[33063, 1354, 211, 5, 42, 33, 0, "StyleData", "DisplayFormula", "All"],
Cell[33277, 1361, 61, 1, 41, 45, 0, "StyleData", "DisplayFormula", 
"Printout"],
Cell[33341, 1364, 95, 2, 42, 53, 0, "StyleData", "DisplayFormula", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[33473, 1371, 389, 12, 39, 30, 0, "StyleData", "DisplayMath", "All",
  CounterIncrements->"Input"],
Cell[33865, 1385, 91, 2, 38, 42, 0, "StyleData", "DisplayMath", "Printout"],
Cell[33959, 1389, 127, 3, 35, 50, 0, "StyleData", "DisplayMath", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[34123, 1397, 59, 1, 36, 33, 0, "StyleData", "EquationNumber", "All"],
Cell[34185, 1400, 86, 2, 35, 45, 0, "StyleData", "EquationNumber", 
"Printout"],
Cell[34274, 1404, 120, 3, 36, 53, 0, "StyleData", "EquationNumber", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[34431, 1412, 166, 4, 52, 26, 0, "StyleData", "Program", "All"],
Cell[34600, 1418, 98, 2, 51, 38, 0, "StyleData", "Program", "Printout"],
Cell[34701, 1422, 134, 3, 50, 46, 0, "StyleData", "Program", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[34884, 1431, 37, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[34946, 1435, 237, 7, 65, 32, 0, "StyleData", "ContentsTitle", "All",
  StyleMenuListing->None],
Cell[35186, 1444, 119, 3, 56, 44, 0, "StyleData", "ContentsTitle", 
"Printout"],
Cell[35308, 1449, 151, 4, 59, 52, 0, "StyleData", "ContentsTitle", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[35496, 1458, 238, 7, 39, 34, 0, "StyleData", "ContentsSection", "All",
  StyleMenuListing->None],
Cell[35737, 1467, 122, 3, 52, 46, 0, "StyleData", "ContentsSection", 
"Printout"],
Cell[35862, 1472, 154, 4, 53, 54, 0, "StyleData", "ContentsSection", 
"EnhancedPrintout"]
}, Open  ]],

Cell[CellGroupData[{
Cell[36053, 1481, 166, 4, 37, 27, 0, "StyleData", "Contents", "All",
  StyleMenuListing->None],
Cell[36222, 1487, 89, 2, 32, 39, 0, "StyleData", "Contents", "Printout"],
Cell[36314, 1491, 123, 3, 33, 47, 0, "StyleData", "Contents", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[36474, 1499, 108, 3, 36, 24, 0, "StyleData", "Index", "All",
  StyleMenuListing->None],
Cell[36585, 1504, 86, 2, 27, 36, 0, "StyleData", "Index", "Printout"],
Cell[36674, 1508, 120, 3, 28, 44, 0, "StyleData", "Index", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[36831, 1516, 116, 3, 36, 32, 0, "StyleData", "IndexSubentry", "All",
  StyleMenuListing->None],
Cell[36950, 1521, 94, 2, 27, 44, 0, "StyleData", "IndexSubentry", 
"Printout"],
Cell[37047, 1525, 128, 3, 28, 52, 0, "StyleData", "IndexSubentry", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[37224, 1534, 32, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[37281, 1538, 263, 8, 42, 31, 0, "StyleData", "FunctionName", "All",
  CellTags->"FunctionName"],
Cell[37547, 1548, 93, 2, 39, 43, 0, "StyleData", "FunctionName", "Printout"],
Cell[37643, 1552, 126, 3, 39, 51, 0, "StyleData", "FunctionName", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[37806, 1560, 155, 4, 36, 38, 0, "StyleData", "FunctionDescription", 
"All",
  CounterIncrements->"Text"],
Cell[37964, 1566, 100, 2, 31, 50, 0, "StyleData", "FunctionDescription", 
"Printout"],
Cell[38067, 1570, 158, 4, 32, 58, 0, "StyleData", "FunctionDescription", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[38262, 1579, 111, 3, 35, 34, 0, "StyleData", "FunctionExample", "All",
  StyleMenuListing->None],
Cell[38376, 1584, 95, 2, 30, 46, 0, "StyleData", "FunctionExample", 
"Printout"],
Cell[38474, 1588, 129, 3, 29, 54, 0, "StyleData", "FunctionExample", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[38652, 1597, 36, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[38713, 1601, 155, 5, 42, 29, 0, "StyleData", "TextNormal", "All",
  CounterIncrements->"Text"],
Cell[38871, 1608, 90, 2, 39, 41, 0, "StyleData", "TextNormal", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[38998, 1615, 299, 9, 40, 35, 0, "StyleData", "BrowserEmulation", "All"],
Cell[39300, 1626, 84, 2, 39, 35, 0, "StyleData", "Text", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[39421, 1633, 259, 8, 56, 31, 0, "StyleData", "CategoryDefs", "All"],
Cell[39683, 1643, 84, 2, 39, 35, 0, "StyleData", "Text", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[39804, 1650, 205, 6, 43, 28, 0, "StyleData", "Category1", "All"],
Cell[40012, 1658, 73, 1, 44, 40, 0, "StyleData", "Category1", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[40122, 1664, 244, 7, 43, 28, 0, "StyleData", "Category2", "All"],
Cell[40369, 1673, 73, 1, 44, 40, 0, "StyleData", "Category2", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[40479, 1679, 242, 7, 41, 28, 0, "StyleData", "Category3", "All"],
Cell[40724, 1688, 73, 1, 42, 40, 0, "StyleData", "Category3", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[40834, 1694, 231, 6, 41, 28, 0, "StyleData", "Category4", "All"],
Cell[41068, 1702, 73, 1, 42, 40, 0, "StyleData", "Category4", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[41190, 1709, 49, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[41264, 1713, 148, 5, 29, 25, 0, "StyleData", "Header", "All",
  StyleMenuListing->None],
Cell[41415, 1720, 37, 0, 29, 37, 0, "StyleData", "Header", "Printout"],
Cell[41455, 1722, 69, 1, 30, 45, 0, "StyleData", "Header", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[41561, 1728, 148, 5, 27, 25, 0, "StyleData", "Footer", "All",
  StyleMenuListing->None],
Cell[41712, 1735, 37, 0, 27, 37, 0, "StyleData", "Footer", "Printout"],
Cell[41752, 1737, 69, 1, 28, 45, 0, "StyleData", "Footer", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[41858, 1743, 152, 5, 32, 29, 0, "StyleData", "PageNumber", "All",
  StyleMenuListing->None],
Cell[42013, 1750, 41, 0, 29, 41, 0, "StyleData", "PageNumber", "Printout"],
Cell[42057, 1752, 73, 1, 30, 49, 0, "StyleData", "PageNumber", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[42179, 1759, 35, 0, 30, "Section"],
Cell[42217, 1761, 184, 4, 46, "Text"],

Cell[CellGroupData[{
Cell[42426, 1769, 302, 8, 36, 28, 0, "StyleData", "Hyperlink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[42731, 1779, 123, 3, 35, 40, 0, "StyleData", "Hyperlink", "Printout"],
Cell[42857, 1784, 157, 4, 36, 48, 0, "StyleData", "Hyperlink", 
"EnhancedPrintout"]
}, Closed]],
Cell[43029, 1791, 104, 3, 27, "Text"],

Cell[CellGroupData[{
Cell[43158, 1798, 316, 8, 36, 31, 0, "StyleData", "MainBookLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[43477, 1808, 126, 3, 35, 43, 0, "StyleData", "MainBookLink", 
"Printout"],
Cell[43606, 1813, 160, 4, 36, 51, 0, "StyleData", "MainBookLink", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[43803, 1822, 337, 9, 36, 29, 0, "StyleData", "AddOnsLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[44143, 1833, 124, 3, 35, 41, 0, "StyleData", "AddOnsLink", "Printout"],
Cell[44270, 1838, 132, 3, 35, 49, 0, "StyleData", "AddOnsLink", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[44439, 1846, 339, 9, 36, 33, 0, "StyleData", "AddOnsLinkText", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[44781, 1857, 128, 3, 35, 45, 0, "StyleData", "AddOnsLinkText", 
"Printout"],
Cell[44912, 1862, 136, 3, 35, 53, 0, "StyleData", "AddOnsLinkText", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[45085, 1870, 341, 9, 36, 31, 0, "StyleData", "RefGuideLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[45429, 1881, 126, 3, 35, 43, 0, "StyleData", "RefGuideLink", 
"Printout"],
Cell[45558, 1886, 162, 4, 34, 51, 0, "StyleData", "RefGuideLink", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[45757, 1895, 328, 8, 36, 37, 0, "StyleData", "GettingStartedLink", 
"All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[46088, 1905, 132, 3, 35, 49, 0, "StyleData", "GettingStartedLink", 
"Printout"],
Cell[46223, 1910, 166, 4, 36, 57, 0, "StyleData", "GettingStartedLink", 
"EnhancedPrintout"]
}, Closed]],

Cell[CellGroupData[{
Cell[46426, 1919, 332, 8, 36, 39, 0, "StyleData", "OtherInformationLink", 
"All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[46761, 1929, 134, 3, 35, 51, 0, "StyleData", "OtherInformationLink", 
"Printout"],
Cell[46898, 1934, 168, 4, 36, 59, 0, "StyleData", "OtherInformationLink", 
"EnhancedPrintout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[47115, 1944, 33, 0, 30, "Section"],

Cell[CellGroupData[{
Cell[47173, 1948, 264, 7, 51, 27, 0, "StyleData", "Outline1", "All",
  CounterIncrements->"Outline1"],
Cell[47440, 1957, 100, 1, 51, 39, 0, "StyleData", "Outline1", "Printout"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47577, 1963, 290, 8, 45, 27, 0, "StyleData", "Outline2", "All",
  CounterIncrements->"Outline2"],
Cell[47870, 1973, 126, 2, 45, 39, 0, "StyleData", "Outline2", "Printout"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48033, 1980, 232, 6, 42, 27, 0, "StyleData", "Outline3", "All",
  CounterIncrements->"Outline3"],
Cell[48268, 1988, 89, 1, 42, 39, 0, "StyleData", "Outline3", "Printout"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48394, 1994, 269, 7, 41, 27, 0, "StyleData", "Outline4", "All",
  CounterIncrements->"Outline4"],
Cell[48666, 2003, 39, 0, 41, 39, 0, "StyleData", "Outline4", "Printout"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

